/**
 * 
 */
package sim.opengl;

import sim.graphics.SColor;
import sim.math.SQuaternion;
import sim.math.SVector3d;
import sim.math.SVectorUV;

import java.util.List;

import com.jogamp.opengl.GL2;

/**
 * La classe <b>SGLDisplay</b> reprsente une classe utilitaire pour faire l'affichage
 * de forme gomtrique  l'aide de l'interface OpenGL sous l'usage de la librairie JOGL.
 *  
 * @author Simon Vzina
 * @since 2016-06-12
 * @version 2023-10-13
 */
public class SGLDisplay {

  /**
   * ...
   */
  public static final double NO_TRANSPARENCY_VALUE = 0.0;
  
  /**
   * ...
   */
  public static final int DEFAULT_SPHERE_DETAIL = 20;
  
  /**
   * ...
   */
  public static final int DEFAULT_DISK_DETAIL = 20;
  
  /**
   * ...
   */
  public static final int DEFAULT_TUBE_DETAIL = 20;
  
  /**
   * ...
   */
  public static final int DEFAULT_CONE_DETAIL = 15;
  
  /**
   * <p>
   * Mthode permettant de dessiner via l'interface OpenGL un <b>cube unitaire</b> centr  l'origine incluant
   * (1) les coordonnes de texture pour 6 faces identique, (2) la normale  la surface des points et (3) la coordonne
   * des points formant les carres du cube. 
   * </p>
   * 
   * <p>
   * Puisque la texture doit pralablement avoir t active si l'on dsire y dessiner un cube textur,
   * les 6 faces auront la mme rfrence  la texture (les 6 ct seront identiques).
   * </p>
   * 
   * @param gl Objet en rfrence au fonctionnalit de openGL.
   */
  public static void displayCube1TextureFace(final GL2 gl)
  {
    // Start Drawing The Cube
    // Systme de coordonne :  x --> vers la droit.
    //                          y --> vers le haut.
    //                          z --> sortant de l'cran
    
    // Start Drawing The Cube
    gl.glBegin(GL2.GL_QUADS); 
         
    // Front Face
    gl.glTexCoord2f(0.0f, 0.0f); gl.glNormal3f(0.0f, 0.0f, 1.0f); gl.glVertex3f(-0.5f, -0.5f, 0.5f);
    gl.glTexCoord2f(1.0f, 0.0f); gl.glNormal3f(0.0f, 0.0f, 1.0f); gl.glVertex3f( 0.5f, -0.5f, 0.5f);
    gl.glTexCoord2f(1.0f, 1.0f); gl.glNormal3f(0.0f, 0.0f, 1.0f); gl.glVertex3f( 0.5f, 0.5f, 0.5f);
    gl.glTexCoord2f(0.0f, 1.0f); gl.glNormal3f(0.0f, 0.0f, 1.0f); gl.glVertex3f(-0.5f, 0.5f, 0.5f);

    // Back Face
    gl.glTexCoord2f(1.0f, 0.0f); gl.glNormal3f(0.0f, 0.0f, -1.0f); gl.glVertex3f(-0.5f, -0.5f, -0.5f);
    gl.glTexCoord2f(1.0f, 1.0f); gl.glNormal3f(0.0f, 0.0f, -1.0f); gl.glVertex3f(-0.5f, 0.5f, -0.5f);
    gl.glTexCoord2f(0.0f, 1.0f); gl.glNormal3f(0.0f, 0.0f, -1.0f); gl.glVertex3f( 0.5f, 0.5f, -0.5f);
    gl.glTexCoord2f(0.0f, 0.0f); gl.glNormal3f(0.0f, 0.0f, -1.0f); gl.glVertex3f( 0.5f, -0.5f, -0.5f);

    // Top Face
    gl.glTexCoord2f(0.0f, 1.0f); gl.glNormal3f(0.0f, 1.0f, 0.0f); gl.glVertex3f(-0.5f, 0.5f, -0.5f);
    gl.glTexCoord2f(0.0f, 0.0f); gl.glNormal3f(0.0f, 1.0f, 0.0f); gl.glVertex3f(-0.5f, 0.5f, 0.5f);
    gl.glTexCoord2f(1.0f, 0.0f); gl.glNormal3f(0.0f, 1.0f, 0.0f); gl.glVertex3f( 0.5f, 0.5f, 0.5f);
    gl.glTexCoord2f(1.0f, 1.0f); gl.glNormal3f(0.0f, 1.0f, 0.0f); gl.glVertex3f( 0.5f, 0.5f, -0.5f);

    // Bottom Face
    gl.glTexCoord2f(1.0f, 1.0f); gl.glNormal3f(0.0f, -1.0f, 0.0f); gl.glVertex3f(-0.5f, -0.5f, -0.5f);
    gl.glTexCoord2f(0.0f, 1.0f); gl.glNormal3f(0.0f, -1.0f, 0.0f); gl.glVertex3f( 0.5f, -0.5f, -0.5f);
    gl.glTexCoord2f(0.0f, 0.0f); gl.glNormal3f(0.0f, -1.0f, 0.0f); gl.glVertex3f( 0.5f, -0.5f, 0.5f);
    gl.glTexCoord2f(1.0f, 0.0f); gl.glNormal3f(0.0f, -1.0f, 0.0f); gl.glVertex3f(-0.5f, -0.5f, 0.5f);

    // Right face
    gl.glTexCoord2f(1.0f, 0.0f); gl.glNormal3f(1.0f, 0.0f, 0.0f); gl.glVertex3f( 0.5f, -0.5f, -0.5f);
    gl.glTexCoord2f(1.0f, 1.0f); gl.glNormal3f(1.0f, 0.0f, 0.0f); gl.glVertex3f( 0.5f, 0.5f, -0.5f);
    gl.glTexCoord2f(0.0f, 1.0f); gl.glNormal3f(1.0f, 0.0f, 0.0f); gl.glVertex3f( 0.5f, 0.5f, 0.5f);
    gl.glTexCoord2f(0.0f, 0.0f); gl.glNormal3f(1.0f, 0.0f, 0.0f); gl.glVertex3f( 0.5f, -0.5f, 0.5f);

    // Left Face
    gl.glTexCoord2f(0.0f, 0.0f); gl.glNormal3f(-1.0f, 0.0f, 0.0f); gl.glVertex3f(-0.5f, -0.5f, -0.5f);
    gl.glTexCoord2f(1.0f, 0.0f); gl.glNormal3f(-1.0f, 0.0f, 0.0f); gl.glVertex3f(-0.5f, -0.5f, 0.5f);
    gl.glTexCoord2f(1.0f, 1.0f); gl.glNormal3f(-1.0f, 0.0f, 0.0f); gl.glVertex3f(-0.5f, 0.5f, 0.5f);
    gl.glTexCoord2f(0.0f, 1.0f); gl.glNormal3f(-1.0f, 0.0f, 0.0f); gl.glVertex3f(-0.5f, 0.5f, -0.5f);
    
    gl.glEnd();   // fin GL_QUADS
    
    gl.glFlush(); // envoyer le tout au pipeline 
  }
  
  /**
   * <p>
   * Mthode permettant de dessiner via l'interface OpenGL un <b>cube unitaire</b> centr  l'origine incluant
   * (1) les coordonnes de texture pour 6 faces diffrences, (2) la normale  la surface des points et (3) la coordonne
   * des points formant les carres du cube. 
   * </p>
   * 
   * <p>
   * Puisque la texture doit pralablement avoir t active si l'on dsire y dessiner un cube textur,
   * les 6 faces auront 6 sections diffrence d'une texture rectangulaire (les 6 ct seront diffrentes).
   * </p>
   * 
   * @param gl Objet en rfrence au fonctionnalit de openGL.
   */
  public static void displayCube6TextureFace(final GL2 gl)
  {
    // Start Drawing The Cube
    // Systme de coordonne :  x --> vers la droit.
    //                          y --> vers le haut.
    //                          z --> sortant de l'cran
        
    // Face #1 : Front Face
    gl.glBegin(GL2.GL_QUADS); 
      
    gl.glTexCoord2f(0.0f, 0.0f);  gl.glNormal3d(0.0f, 0.0f, 1.0f); gl.glVertex3f(-0.5f, -0.5f, 0.5f);
    gl.glTexCoord2f(0.33f, 0.0f); gl.glNormal3d(0.0f, 0.0f, 1.0f); gl.glVertex3f( 0.5f, -0.5f, 0.5f);
    gl.glTexCoord2f(0.33f, 0.5f); gl.glNormal3d(0.0f, 0.0f, 1.0f); gl.glVertex3f( 0.5f, 0.5f, 0.5f);
    gl.glTexCoord2f(0.0f, 0.5f);  gl.glNormal3d(0.0f, 0.0f, 1.0f); gl.glVertex3f(-0.5f, 0.5f, 0.5f);

    gl.glEnd();
    gl.glFlush();
        
    // Face #2 : Back Face
    gl.glBegin(GL2.GL_QUADS); 
       
    gl.glTexCoord2f(0.33f, 0.0f); gl.glNormal3d(0.0f, 0.0f, -1.0f); gl.glVertex3f(-0.5f, -0.5f, -0.5f);
    gl.glTexCoord2f(0.66f, 0.0f); gl.glNormal3d(0.0f, 0.0f, -1.0f); gl.glVertex3f(-0.5f, 0.5f, -0.5f);
    gl.glTexCoord2f(0.66f, 0.5f); gl.glNormal3d(0.0f, 0.0f, -1.0f); gl.glVertex3f( 0.5f, 0.5f, -0.5f);
    gl.glTexCoord2f(0.33f, 0.5f); gl.glNormal3d(0.0f, 0.0f, -1.0f); gl.glVertex3f( 0.5f, -0.5f, -0.5f);

    gl.glEnd();
    gl.glFlush();
    
    // Face #3 : Top Face
    gl.glBegin(GL2.GL_QUADS); 
    
    gl.glTexCoord2f(0.66f, 0.0f); gl.glNormal3d(0.0f, 1.0f, 0.0f); gl.glVertex3f(-0.5f, 0.5f, -0.5f);
    gl.glTexCoord2f(1.0f, 0.0f);  gl.glNormal3d(0.0f, 1.0f, 0.0f); gl.glVertex3f(-0.5f, 0.5f, 0.5f);
    gl.glTexCoord2f(1.0f, 0.5f);  gl.glNormal3d(0.0f, 1.0f, 0.0f); gl.glVertex3f( 0.5f, 0.5f, 0.5f);
    gl.glTexCoord2f(0.66f, 0.5f); gl.glNormal3d(0.0f, 1.0f, 0.0f); gl.glVertex3f( 0.5f, 0.5f, -0.5f);

    gl.glEnd();
    gl.glFlush();
    
       
    // Face #4 : Bottom Face
    gl.glBegin(GL2.GL_QUADS); 
            
    gl.glTexCoord2f(0.0f, 0.5f);  gl.glNormal3d(0.0f, -1.0f, 0.0f); gl.glVertex3f(-0.5f, -0.5f, -0.5f);
    gl.glTexCoord2f(0.33f, 0.5f); gl.glNormal3d(0.0f, -1.0f, 0.0f); gl.glVertex3f( 0.5f, -0.5f, -0.5f);
    gl.glTexCoord2f(0.33f, 1.0f); gl.glNormal3d(0.0f, -1.0f, 0.0f); gl.glVertex3f( 0.5f, -0.5f, 0.5f);
    gl.glTexCoord2f(0.0f, 1.0f);  gl.glNormal3d(0.0f, -1.0f, 0.0f); gl.glVertex3f(-0.5f, -0.5f, 0.5f);

    gl.glEnd();
    gl.glFlush();
    
    // Face #5 : Right face
    gl.glBegin(GL2.GL_QUADS);
        
    gl.glTexCoord2f(0.33f, 0.5f); gl.glNormal3d(1.0f, 0.0f, 0.0f); gl.glVertex3f( 0.5f, -0.5f, -0.5f);
    gl.glTexCoord2f(0.66f, 0.5f); gl.glNormal3d(1.0f, 0.0f, 0.0f); gl.glVertex3f( 0.5f, 0.5f, -0.5f);
    gl.glTexCoord2f(0.66f, 1.0f); gl.glNormal3d(1.0f, 0.0f, 0.0f); gl.glVertex3f( 0.5f, 0.5f, 0.5f);
    gl.glTexCoord2f(0.33f, 1.0f); gl.glNormal3d(1.0f, 0.0f, 0.0f); gl.glVertex3f( 0.5f, -0.5f, 0.5f);

    gl.glEnd();
    gl.glFlush();
    
    // Face #6 : Left Face    
    gl.glBegin(GL2.GL_QUADS);
        
    gl.glTexCoord2f(0.66f, 0.5f); gl.glNormal3d(-1.0f, 0.0f, 0.0f); gl.glVertex3f(-0.5f, -0.5f, -0.5f);
    gl.glTexCoord2f(1.0f, 0.5f);  gl.glNormal3d(-1.0f, 0.0f, 0.0f); gl.glVertex3f(-0.5f, -0.5f, 0.5f);
    gl.glTexCoord2f(1.0f, 1.0f);  gl.glNormal3d(-1.0f, 0.0f, 0.0f); gl.glVertex3f(-0.5f, 0.5f, 0.5f);
    gl.glTexCoord2f(0.66f, 1.0f); gl.glNormal3d(-1.0f, 0.0f, 0.0f); gl.glVertex3f(-0.5f, 0.5f, -0.5f);
    
    gl.glEnd();
    gl.glFlush();
  }
  
  /**
   * Mthode permettant de dessiner via l'interface OpenGL un <b>cube unitaire</b> avec coordonne de texture.
   * 
   * @param gl Objet en rfrence au fonctionnalit de openGL.
   * @param gl_texture_id Le tableau des numros d'accs aux textures  appliquer sur les 6 faces du cube.
   * @throws IllegalArgumentException Si le tableau d'accs au numro de texture n'est pas de taille 6.
   */
  public static void displayCube(final GL2 gl, int[] gl_texture_id) throws IllegalArgumentException
  {
    // Validation.
    if(gl_texture_id.length != 6)
      throw new IllegalArgumentException("Erreur SGLGeometry 001 : Le tableau d'accs au numro de texture est de taille gal  " + gl_texture_id.length + " ce qui n'est pas gal  6.");
    
    // Affectation de couleur de fond blanc pour l'addition avec la couleur de la texture.
    gl.glColor3d(1.0f, 1.0f, 1.0f);
       
    // Activation des texture 2d.
    gl.glEnable(GL2.GL_TEXTURE_2D);
    
    // Accs  la texture #1.
    gl.glBindTexture(GL2.GL_TEXTURE_2D, gl_texture_id[0]);   
    
    // Start Drawing The Cube
    gl.glBegin(GL2.GL_QUADS); 
    
    // Front Face
    gl.glTexCoord2f(0.0f, 0.0f); gl.glVertex3f(-0.5f, -0.5f, 0.5f);
    gl.glTexCoord2f(1.0f, 0.0f); gl.glVertex3f( 0.5f, -0.5f, 0.5f);
    gl.glTexCoord2f(1.0f, 1.0f); gl.glVertex3f( 0.5f, 0.5f, 0.5f);
    gl.glTexCoord2f(0.0f, 1.0f); gl.glVertex3f(-0.5f, 0.5f, 0.5f);

    gl.glEnd();
    gl.glFlush();
    
    // Accs  la texture #2.
    gl.glBindTexture(GL2.GL_TEXTURE_2D, gl_texture_id[1]);
    
    gl.glBegin(GL2.GL_QUADS); 
    
    // Back Face
    gl.glTexCoord2f(1.0f, 0.0f); gl.glVertex3f(-0.5f, -0.5f, -0.5f);
    gl.glTexCoord2f(1.0f, 1.0f); gl.glVertex3f(-0.5f, 0.5f, -0.5f);
    gl.glTexCoord2f(0.0f, 1.0f); gl.glVertex3f( 0.5f, 0.5f, -0.5f);
    gl.glTexCoord2f(0.0f, 0.0f); gl.glVertex3f( 0.5f, -0.5f, -0.5f);

    gl.glEnd();
    gl.glFlush();
    
    // Accs  la texture #3.
    gl.glBindTexture(GL2.GL_TEXTURE_2D, gl_texture_id[2]);
    
    gl.glBegin(GL2.GL_QUADS); 
    
    // Top Face
    gl.glTexCoord2f(0.0f, 1.0f); gl.glVertex3f(-0.5f, 0.5f, -0.5f);
    gl.glTexCoord2f(0.0f, 0.0f); gl.glVertex3f(-0.5f, 0.5f, 0.5f);
    gl.glTexCoord2f(1.0f, 0.0f); gl.glVertex3f( 0.5f, 0.5f, 0.5f);
    gl.glTexCoord2f(1.0f, 1.0f); gl.glVertex3f( 0.5f, 0.5f, -0.5f);

    gl.glEnd();
    gl.glFlush();
    
    // Accs  la texture #4.
    gl.glBindTexture(GL2.GL_TEXTURE_2D, gl_texture_id[3]);
    
    gl.glBegin(GL2.GL_QUADS); 
    
    // Bottom Face
    gl.glTexCoord2f(1.0f, 1.0f); gl.glVertex3f(-0.5f, -0.5f, -0.5f);
    gl.glTexCoord2f(0.0f, 1.0f); gl.glVertex3f( 0.5f, -0.5f, -0.5f);
    gl.glTexCoord2f(0.0f, 0.0f); gl.glVertex3f( 0.5f, -0.5f, 0.5f);
    gl.glTexCoord2f(1.0f, 0.0f); gl.glVertex3f(-0.5f, -0.5f, 0.5f);

    gl.glEnd();
    gl.glFlush();
    
    // Accs  la texture #5.
    gl.glBindTexture(GL2.GL_TEXTURE_2D, gl_texture_id[4]);
    
    gl.glBegin(GL2.GL_QUADS);
    
    // Right face
    gl.glTexCoord2f(1.0f, 0.0f); gl.glVertex3f( 0.5f, -0.5f, -0.5f);
    gl.glTexCoord2f(1.0f, 1.0f); gl.glVertex3f( 0.5f, 0.5f, -0.5f);
    gl.glTexCoord2f(0.0f, 1.0f); gl.glVertex3f( 0.5f, 0.5f, 0.5f);
    gl.glTexCoord2f(0.0f, 0.0f); gl.glVertex3f( 0.5f, -0.5f, 0.5f);

    gl.glEnd();
    gl.glFlush();
    
    // Accs  la texture #6.
    gl.glBindTexture(GL2.GL_TEXTURE_2D, gl_texture_id[5]);
    
    gl.glBegin(GL2.GL_QUADS);
    
    // Left Face
    gl.glTexCoord2f(0.0f, 0.0f); gl.glVertex3f(-0.5f, -0.5f, -0.5f);
    gl.glTexCoord2f(1.0f, 0.0f); gl.glVertex3f(-0.5f, -0.5f, 0.5f);
    gl.glTexCoord2f(1.0f, 1.0f); gl.glVertex3f(-0.5f, 0.5f, 0.5f);
    gl.glTexCoord2f(0.0f, 1.0f); gl.glVertex3f(-0.5f, 0.5f, -0.5f);
    
    gl.glEnd();
    gl.glFlush();
    
    // Dsactivation des textures.
    gl.glDisable(GL2.GL_TEXTURE_2D);
  }
    
  /**
   * Mthode permettant de dessiner via l'interface OpenGL un <b>cube unitaire centr  l'origine</b> avec coordonne de texture pour <u>une texture contenant les 6 faces du cube</u>.
   * 
   * @param gl Objet en rfrence au fonctionnalit de openGL.
   * @param gl_texture_id Le numros d'accs  la texture  appliquer sur les 6 faces du cube.
   */
  public static void displayCube(final GL2 gl, int gl_texture_id, double transparency)
  {
    // Activer le mlange des couleurs.
    gl.glEnable(GL2.GL_BLEND);
    
    // Choisir la fonction de mlange des couleurs.
    gl.glBlendFunc(GL2.GL_SRC_ALPHA, GL2.GL_ONE_MINUS_SRC_ALPHA);
   
    // Dfinir la couleur de base avec transparence. La couleur de la texture se superposera  celle-ci.
    // Remarque : Si c'est opaque, le alpha doit tre gale  1.0.
    gl.glColor4d(1.0, 1.0, 1.0, 1.0 - transparency);
    
    // S'il y a de la transparence, il faut bloque de z-buffer de profondeur
    // pour ainsi pouvoir mlanger les couleurs.
    if(transparency != 0.0)
      gl.glDepthMask(false); 
    
    // Activation des texture 2d.
    gl.glEnable(GL2.GL_TEXTURE_2D);
    
    // Accs  la texture.
    gl.glBindTexture(GL2.GL_TEXTURE_2D, gl_texture_id);   
    
    //--------------------------------------------------
    // Dessiner le cube unitaire avec texture  6 faces.
    //--------------------------------------------------
    displayCube6TextureFace(gl);
    
    // Dsactiver les textures.
    gl.glDisable(GL2.GL_TEXTURE_2D);
    
    // Remettre comme avant.
    if(transparency != 0.0)
      gl.glDepthMask(true);
    
    // Remettre comme avant.
    gl.glDisable(GL2.GL_BLEND);
  }
  
  /**
   * ...
   * 
   * @param gl
   * @param color
   */
  public static void displayCube(final GL2 gl, SColor color)
  {
    //Initialiser la couleur du cube
    SGL.setColor(gl, color);
    
    // Dessiner le cube avec les transformations appropries
    displayCube1TextureFace(gl);
  }
   
  /**
   * ...
   * 
   * @param gl
   * @param scale
   * @param rotation
   * @param translation
   */
  public static void displayCube(final GL2 gl, SColor color, SVector3d scale, SQuaternion rotation, SVector3d translation)
  {
    // Dfinir les matrices de transformation de modle
    gl.glMatrixMode(GL2.GL_MODELVIEW);
    
    // Transformer le futur cube
    SGL.pushTransformationMatrix(gl, scale, rotation, translation);
    
    // Dessiner le cube unitaire
    displayCube(gl, color);
    
    // Remettre les matrices de transformation d'origine
    gl.glPopMatrix();
  }
    
  /**
   * ...
   * 
   * @param gl
   * @param gl_texture_id
   * @param scale
   * @param rotation
   * @param translation
   */
  public static void displayCube(final GL2 gl, int gl_texture_id, SVector3d scale, SQuaternion rotation, SVector3d translation)
  {
    displayCube(gl, gl_texture_id, NO_TRANSPARENCY_VALUE, scale, rotation, translation);
  }
  
  /**
   * Mthode pour dessiner un cube avec texture et niveau de transparence.
   * 
   * @param gl
   * @param gl_texture_id
   * @param transparency La transparence (0 : opaque, 1 : transparent)
   * @param scale
   * @param rotation
   * @param translation
   */
  public static void displayCube(final GL2 gl, int gl_texture_id, double transparency, SVector3d scale, SQuaternion rotation, SVector3d translation)
  {
    // Dfinir les matrices de transformation de modle
    gl.glMatrixMode(GL2.GL_MODELVIEW);
    
    // Transformer le futur cube
    SGL.pushTransformationMatrix(gl, scale, rotation, translation);
    
    // Dessiner le cube.
    displayCube(gl, gl_texture_id, transparency);
    
    // Remettre les matrices de transformation d'origine
    gl.glPopMatrix();
  }
  
  /**
   * Mthode permettant de dessiner un cube.
   * 
   * @param gl Objet en rfrence au fonctionnalit de openGL.
   * @param color La couleur du cube.
   * @param scale La taille du cube.
   * @param rotation L'tat de rotation Rxyz du cube.
   * @param translation La position du cube.
   */
  public static void displayCube(final GL2 gl, SColor color, SVector3d scale, SVector3d rotation, SVector3d translation)
  {
    // Dfinir les matrices de transformation de modle
    gl.glMatrixMode(GL2.GL_MODELVIEW);
    
    // Transformer le futur cube
    SGL.pushTransformationMatrix(gl, scale, rotation, translation);
    
    // Dessiner le cube unitaire
    displayCube(gl, color);
    
    // Remettre les matrices de transformation d'origine
    gl.glPopMatrix();
  }
    
  /**
   * Mthode permettant de dessiner via l'interface OpenGL un <b>tetrade centr  l'origine de segment de 1.633 unit</b> avec coordonne de texture pour <u>une texture contenant les 4 triangles</u>.
   * 
   * @param gl
   */
  public static void displayTetrahedron(final GL2 gl, int gl_texture_id)
  {
    // Affectation de couleur de fond blanc pour l'addition avec la couleur de la texture.
    gl.glColor3d(1.0f, 1.0f, 1.0f);
       
    // Activation des texture 2d.
    gl.glEnable(GL2.GL_TEXTURE_2D);
    
    // Accs  la texture.
    gl.glBindTexture(GL2.GL_TEXTURE_2D, gl_texture_id); 
    
    // Les 4 points du Tetrade : 
    // (0.9428f, 0.0f, -0.3333f)     correspondant  :  Math.sqrt(8.0/9.0), 0.0, -1.0/3.0
    // (-0.4714f, 0.8165f, -0.3333f)  correspondant  : -Math.sqrt(2.0/9.0), Math.sqrt(2.0/3.0), -1.0/3.0
    // (-0.4714f, -0.8165f, -0.3333f) correspondant  : -Math.sqrt(2.0/9.0), -Math.sqrt(2.0/3.0), -1.0/3.0
    // (0.0f, 0.0f, 1.0f)

    // Le triangle dans le plan XY : Point 0, 2, 1
    gl.glBegin(GL2.GL_TRIANGLES);
    
    gl.glTexCoord2f(0.0f, 0.0f); gl.glNormal3d(0.0f, 0.0f, -1.0f); gl.glVertex3d(0.9428f, 0.0f, -0.3333f);     
    gl.glTexCoord2f(0.4f, 0.0f); gl.glNormal3d(0.0f, 0.0f, -1.0f); gl.glVertex3d(-0.4714f, -0.8165f, -0.3333f);
    gl.glTexCoord2f(0.2f, 1.0f); gl.glNormal3d(0.0f, 0.0f, -1.0f); gl.glVertex3d(-0.4714f, 0.8165f, -0.3333f);  
    
    gl.glEnd();
    gl.glFlush();
    
    // Le 1er triangle de la pyramide : Point 0, 1, 3
    gl.glBegin(GL2.GL_TRIANGLES);
    
    gl.glTexCoord2f(0.2f, 0.0f);  gl.glVertex3d(0.9428f, 0.0f, -0.33333f);     
    gl.glTexCoord2f(0.6f, 1.0f);  gl.glVertex3d(-0.4714f, 0.8165f, -0.3333f);
    gl.glTexCoord2f(0.2f, 1.0f);  gl.glVertex3d(0.0f, 0.0f, 1.0f);  
    
    gl.glEnd();
    gl.glFlush();
    
    // Le 2e triangle de la pyramide : Point 1, 2, 3
    gl.glBegin(GL2.GL_TRIANGLES);
    
    gl.glTexCoord2f(0.4f, 0.0f);  gl.glVertex3d(-0.4714f, 0.8165f, -0.3333f);     
    gl.glTexCoord2f(0.8f, 0.0f);  gl.glVertex3d(-0.4714f, -0.8165f, -0.3333f);
    gl.glTexCoord2f(0.6f, 1.0f);  gl.glVertex3d(0.0f, 0.0f, 1.0f);  
    
    gl.glEnd();
    gl.glFlush();
    
    // Le 3e triangle de la pyramide : Point 0, 3, 2
    gl.glBegin(GL2.GL_TRIANGLES);
    
    gl.glTexCoord2f(0.8f, 0.0f);  gl.glVertex3d(0.9428f, 0.0f, -0.33333f);     
    gl.glTexCoord2f(1.0f, 1.0f);  gl.glVertex3d(0.0f, 0.0f, 1.0f);
    gl.glTexCoord2f(0.6f, 1.0f);  gl.glVertex3d(-0.4714f, -0.8165f, -0.3333f);  
    
    gl.glEnd();
    gl.glFlush();
    
    // Dsactivation des textures.
    gl.glDisable(GL2.GL_TEXTURE_2D);
  }
  
  /**
   * ...
   * 
   * @param gl
   * @param gl_texture_id
   * @param scale
   * @param rotation
   * @param translation
   */
  public static void displayTetrahedron(final GL2 gl, int gl_texture_id, SVector3d scale, SQuaternion rotation, SVector3d translation)
  {
    // Dfinir les matrices de transformation de modle
    gl.glMatrixMode(GL2.GL_MODELVIEW);
    
    // Transformer le futur cube
    SGL.pushTransformationMatrix(gl, scale, rotation, translation);
    
    // Dessiner le cube unitaire
    displayTetrahedron(gl, gl_texture_id);
    
    // Remettre les matrices de transformation d'origine
    gl.glPopMatrix();
  }
  
  /**
   * Mthode permettant de dessiner une sphere positionne en son centre.
   * 
   * @param gl Objet en rfrence au fonctionnalit de openGL.
   * @param detail Le niveau de dtail de la sphre influencant le nombre de carrs ncessaire pour dessiner la sphre.
   */
  public static void displaySphere(final GL2 gl, int detail)
  {
    if(detail < DEFAULT_SPHERE_DETAIL)
      detail = DEFAULT_SPHERE_DETAIL;
    
    float x,y,z;
    float theta1, theta2, theta3;

    final float PI_VALUE = (float) Math.PI;
        
    for(int j=0; j <= detail/2 - 1; j++)
    {
      theta1 = j*2.0f*PI_VALUE/detail - PI_VALUE/2.0f;
      theta2 = (j+1)*2.0f*PI_VALUE/detail - PI_VALUE/2.0f;

      gl.glBegin(GL2.GL_QUAD_STRIP);
        for(int i=0; i<=detail; i++)
        {
          theta3 = i*2.0f*PI_VALUE/detail;
          x = (float) (Math.cos(theta2)*Math.cos(theta3));
          y = (float) Math.sin(theta2);
          z = (float) (Math.cos(theta2)*Math.sin(theta3));

          gl.glNormal3f(x,y,z);  
          gl.glTexCoord2f(1.0f-(float)i/detail, 2.0f*(float)(j+1)/detail);     
          gl.glVertex3f(x,y,z);

          x = (float) (Math.cos(theta1)*Math.cos(theta3));
          y = (float) Math.sin(theta1);
          z = (float) (Math.cos(theta1)*Math.sin(theta3));

          gl.glNormal3f(x,y,z);  
          gl.glTexCoord2f(1.0f-(float)i/detail, 2.0f*(float)j/detail);         
          gl.glVertex3f(x,y,z);
        }
      gl.glEnd();
    }

    gl.glFlush();
  }
  
  /**
   * Mthode permettant de dessiner via l'interface OpenGL un <b>cube unitaire centr  l'origine</b> avec coordonne de texture pour <u>une texture contenant les 6 faces du cube</u>.
   * 
   * @param gl Objet en rfrence au fonctionnalit de openGL.
   * @param gl_texture_id Le numros d'accs  la texture  appliquer sur les 6 faces du cube.
   */
  public static void displaySphere(final GL2 gl, int gl_texture_id, double transparency, SColor color)
  {
    // Activation des texture 2d.
    gl.glEnable(GL2.GL_TEXTURE_2D);
    
    // Accs  la texture.
    gl.glBindTexture(GL2.GL_TEXTURE_2D, gl_texture_id);
    
    // S'il n'y a pas de transparence
    if(transparency == 0.0)
    {
      // Dfinir la couleur.
      SGL.setColor(gl, color);
      
      //--------------------
      // Dessiner la sphre.
      //--------------------
      displaySphere(gl, DEFAULT_SPHERE_DETAIL);
    }
    else
    {
      // Activer le mlange des couleurs.
      gl.glEnable(GL2.GL_BLEND);
      
      // Choisir la fonction de mlange des couleurs.
      gl.glBlendFunc(GL2.GL_SRC_ALPHA, GL2.GL_ONE_MINUS_SRC_ALPHA);
     
      // Dfinir la couleur de base avec transparence. La couleur de la texture se superposera  celle-ci.
      // Remarque : Si c'est opaque, le alpha doit tre gale  1.0.
      gl.glColor4d(color.getRed(), color.getGreen(), color.getBlue(), 1.0 - transparency);
      
      // S'il y a de la transparence, il faut bloque de z-buffer de profondeur
      // pour ainsi pouvoir mlanger les couleurs.
      gl.glDepthMask(false); 
            
      //--------------------
      // Dessiner la sphre.
      //--------------------
      displaySphere(gl, DEFAULT_SPHERE_DETAIL);
            
      // Remettre comme avant.
      gl.glDepthMask(true);
      
      // Remettre comme avant.
      gl.glDisable(GL2.GL_BLEND);
    }
    
    // Dsactiver les textures.
    gl.glDisable(GL2.GL_TEXTURE_2D);
  }
  
  /**
   * ...
   * 
   * @param gl
   * @param transparency
   * @param color
   */
  public static void displaySphere(final GL2 gl, double transparency, SColor color)
  {
    // S'il n'y a pas de transparence
    if(transparency == 0.0)
    {
      // Dfinir la couleur.
      SGL.setColor(gl, color);
      
      //--------------------
      // Dessiner la sphre.
      //--------------------
      displaySphere(gl, DEFAULT_SPHERE_DETAIL);
    }
    else
    {
      // Activer le mlange des couleurs.
      gl.glEnable(GL2.GL_BLEND);
      
      // Choisir la fonction de mlange des couleurs.
      gl.glBlendFunc(GL2.GL_SRC_ALPHA, GL2.GL_ONE_MINUS_SRC_ALPHA);
     
      // Dfinir la couleur de base avec transparence. La couleur de la texture se superposera  celle-ci.
      // Remarque : Si c'est opaque, le alpha doit tre gale  1.0.
      gl.glColor4d(color.getRed(), color.getGreen(), color.getBlue(), 1.0 - transparency);
      
      // S'il y a de la transparence, il faut bloque de z-buffer de profondeur
      // pour ainsi pouvoir mlanger les couleurs.
      gl.glDepthMask(false); 
      
      //--------------------------------------------------
      // Dessiner le cube unitaire avec texture  6 faces.
      //--------------------------------------------------
      displaySphere(gl, DEFAULT_SPHERE_DETAIL);
      
      // Remettre comme avant.
      gl.glDepthMask(true);
      
      // Remettre comme avant.
      gl.glDisable(GL2.GL_BLEND);
    }
  }
  
  /**
   * ...
   * 
   * @param gl
   * @param gl_texture_id
   * @param transparency
   */
  public static void displaySphere(final GL2 gl, int gl_texture_id, double transparency)
  {
    displaySphere(gl, gl_texture_id, transparency, SColor.WHITE);
  }
    
  /**
   * Mthode permettant de dessiner une sphere positionne en son centre.
   * 
   * @param gl Objet en rfrence au fonctionnalit de openGL.
   * @param color La couleur de la sphre.
   * @param scale La taille de la sphre (elle peut tre dform par le scale).
   * @param rotation Lae rotation Rxyz de la sphre.
   * @param translation La position de la sphre.
   */
  public static void displaySphere(final GL2 gl, double transparency, SColor color, SVector3d scale, SVector3d rotation, SVector3d translation)
  {
    // Dfinir les matrices de transformation de modle
    gl.glMatrixMode(GL2.GL_MODELVIEW);
    
    // Transformer la futur sphre.
    SGL.pushTransformationMatrix(gl, scale, rotation, translation);
    
    // Dessiner le cube unitaire
    displaySphere(gl, transparency, color);
    
    // Remettre les matrices de transformation d'origine
    gl.glPopMatrix();
  }
    
  /**
   * ...
   * 
   * @param gl
   * @param color
   * @param scale
   * @param rotation
   * @param translation
   */
  public static void displaySphere(final GL2 gl, SColor color, SVector3d scale, SVector3d rotation, SVector3d translation)
  {
    displaySphere(gl, NO_TRANSPARENCY_VALUE, color, scale, rotation, translation);
  }
  
  /**
   * ...
   * 
   * @param gl
   * @param gl_texture_id
   * @param transparency
   * @param scale
   * @param rotation
   * @param translation
   */
  public static void displaySphere(final GL2 gl, int gl_texture_id, double transparency, SVector3d scale, SQuaternion rotation, SVector3d translation)
  {
    // Dfinir les matrices de transformation de modle
    gl.glMatrixMode(GL2.GL_MODELVIEW);
    
    // Transformer le futur cube
    SGL.pushTransformationMatrix(gl, scale, rotation, translation);
    
    // Dessiner le cube.
    displaySphere(gl, gl_texture_id, transparency);
    
    // Remettre les matrices de transformation d'origine
    gl.glPopMatrix();
  }
  
  /**
   * ...
   * 
   * @param gl
   * @param gl_texture_id
   * @param transparency
   * @param scale
   * @param rotation
   * @param translation
   */
  public static void displaySphere(final GL2 gl, int gl_texture_id, double transparency, SVector3d scale, SVector3d rotation, SVector3d translation)
  {
    // Dfinir les matrices de transformation de modle
    gl.glMatrixMode(GL2.GL_MODELVIEW);
    
    // Transformer le futur cube
    SGL.pushTransformationMatrix(gl, scale, rotation, translation);
    
    // Dessiner le cube.
    displaySphere(gl, gl_texture_id, transparency);
    
    // Remettre les matrices de transformation d'origine
    gl.glPopMatrix();
  }
  
  /**
   * Mthode pour dessiner un disque dans le plan xy centr  l'origine.
   * 
   * @param gl
   * @param r
   * @param normal_up
   * @param detail
   */
  public static void displayDisk(GL2 gl, double r, boolean normal_up, int detail)
  {
    if(r < 0.0)
      r*= -1.0;
    
    // Dfinir le sens de la normale.
    double n_z;
    
    if(normal_up)
      n_z = 1.0;
    else
      n_z = -1.0;
    
    if(detail < DEFAULT_DISK_DETAIL)
      detail = DEFAULT_DISK_DETAIL;
    
    double dtheta = 2.0*Math.PI / detail;
    double du = 1.0 / detail;
    double  x, y,
            u = 0.0, theta = 0.0;

    gl.glBegin(GL2.GL_TRIANGLE_FAN);
      gl.glNormal3d(0.0, 0.0, n_z);  gl.glTexCoord2d(u, 1.0);  gl.glVertex3d(0.0, 0.0, 0.0);
      
    // Itration sur l'ensemble des rectangles qui vont dfinir le tube.
    for(int i = 0; i <= detail; i++)
    {
      theta = dtheta*i;
      u = du*i;
     
      x = r*Math.cos(theta);
      y = r*Math.sin(theta);
     
      gl.glNormal3d(0.0, 0.0, n_z);  gl.glTexCoord2d(u, 1.0);  gl.glVertex3d(x, y, 0.0);
            
    }
    
    gl.glEnd();
  }
  
  /**
   * Mthode pour dessiner un disque dans le plan xy centr  l'origine dont la normale est oriente vers le haut (en z positif).
   * 
   * @param gl
   * @param detail
   * @param r
   * @param normal_up
   */
  public static void displayDisk(GL2 gl)
  {
    displayDisk(gl, 1.0, true, DEFAULT_DISK_DETAIL);
  }
  
  /**
   * ...
   * 
   * @param gl
   * @param transparency
   * @param color
   */
  public static void displayDisk(final GL2 gl, double transparency, SColor color)
  {
    // S'il n'y a pas de transparence
    if(transparency == 0.0)
    {
      SGL.setColor(gl, color);
      displayDisk(gl);
    }
    else
    {
      // Activer le mlange des couleurs.
      gl.glEnable(GL2.GL_BLEND);
      
      // Choisir la fonction de mlange des couleurs.
      gl.glBlendFunc(GL2.GL_SRC_ALPHA, GL2.GL_ONE_MINUS_SRC_ALPHA);
     
      // Dfinir la couleur de base avec transparence. La couleur de la texture se superposera  celle-ci.
      // Remarque : Si c'est opaque, le alpha doit tre gale  1.0.
      gl.glColor4d(color.getRed(), color.getGreen(), color.getBlue(), 1.0 - transparency);
      
      // S'il y a de la transparence, il faut bloque de z-buffer de profondeur
      // pour ainsi pouvoir mlanger les couleurs.
      gl.glDepthMask(false); 
      
      //--------------------------------------------------
      // Dessiner le cube unitaire avec texture  6 faces.
      //--------------------------------------------------
      displayDisk(gl);
      
      // Remettre comme avant.
      gl.glDepthMask(true);
      
      // Remettre comme avant.
      gl.glDisable(GL2.GL_BLEND);
    }
  }
  
  /**
   * ...
   * 
   * @param gl
   * @param transparency
   * @param color
   * @param scale
   * @param rotation
   * @param translation
   */
  public static void displayDisk(final GL2 gl, double transparency, SColor color, SVector3d scale, SVector3d rotation, SVector3d translation)
  {
    // Dfinir les matrices de transformation de modle
    gl.glMatrixMode(GL2.GL_MODELVIEW);
    
    // Transformer la futur sphre.
    SGL.pushTransformationMatrix(gl, scale, rotation, translation);
    
    // Dessiner le cube unitaire
    displayDisk(gl, transparency, color);
    
    // Remettre les matrices de transformation d'origine
    gl.glPopMatrix();
  }
    
  /**
   * ...
   * 
   * @param gl
   * @param color
   * @param scale
   * @param rotation
   * @param translation
   */
  public static void displayDisk(final GL2 gl, SColor color, SVector3d scale, SVector3d rotation, SVector3d translation)
  {
    displayDisk(gl, NO_TRANSPARENCY_VALUE, color, scale, rotation, translation);
  }
  
  /**
   * Mthode pour dessiner un tube unitaire align le long de l'axe z.
   * 
   * @param gl
   * @param detail
   */
  public static void displayTube(GL2 gl, int detail)
  {
    if(detail < DEFAULT_TUBE_DETAIL)
      detail = DEFAULT_TUBE_DETAIL;
    
    double dtheta = 2.0*Math.PI / detail;
    double du = 1.0 / detail;
    double  x0 = 0.0, y0 = 0.0,
            x1 = 0.0, y1 = 0.0,
            u0 = 0.0, u1 = 0.0,
            theta0 = 0.0, theta1 = 0.0;

    // Itration sur l'ensemble des rectangles qui vont dfinir le tube.
    for(int i = 0; i < detail; i++)
    {
      theta0 = theta1;
      u0 = u1;
      theta1 += dtheta;
      u1 += du;

      x0 = Math.cos(theta0);
      y0 = Math.sin(theta0);
      x1 = Math.cos(theta1);
      y1 = Math.sin(theta1);

      //  Dessiner un petit rectangle avec un section de la texture //
      gl.glBegin(GL2.GL_QUADS);
        gl.glNormal3d(x0,y0,0.0);  gl.glTexCoord2d(u0, 1.0);  gl.glVertex3d(x0, y0, 1.0);
        gl.glNormal3d(x0,y0,0.0);  gl.glTexCoord2d(u0, 0.0);  gl.glVertex3d(x0, y0, 0.0);
        gl.glNormal3d(x1,y1,0.0);  gl.glTexCoord2d(u1, 0.0);  gl.glVertex3d(x1, y1, 0.0);
        gl.glNormal3d(x1,y1,0.0);  gl.glTexCoord2d(u1, 1.0);  gl.glVertex3d(x1, y1, 1.0);
      gl.glEnd();
    }
  }
  
  /**
   * Mthode permettant de dessiner un cylindre.
   * 
   * @param gl Objet en rfrence au fonctionnalit de openGL.
   * @param color La couleur du cylindre.
   * @param scale La taille du cylindre.
   * @param rotation L'tat de rotation Rxyz du cylindre.
   * @param translation La position du cylindre.
   */
  public static void displayCylinder(final GL2 gl, SColor color, SVector3d scale, SVector3d rotation, SVector3d translation)
  {
    // Initialiser la couleur du cube
    SGL.setColor(gl, color);
    
    // Dessiner le cube avec les transformations appropries
    displayCylinder(gl, scale, rotation, translation);
  }
  
  /**
   * Mthode permettant de dessiner un cone.
   * 
   * @param gl Objet en rfrence au fonctionnalit de openGL.
   * @param color La couleur du cone.
   * @param scale La taille du cone.
   * @param rotation L'tat de rotation Rxyz du cone.
   * @param translation La position du cone.
   */
  public static void displayCone(final GL2 gl, SColor color, SVector3d scale, SVector3d rotation, SVector3d translation)
  {
    // Initialiser la couleur du cube
    SGL.setColor(gl, color);
    
    // Dessiner le cube avec les transformations appropries
    displayCone(gl, scale, rotation, translation);
  }
  
  /**
   * Mthode pour dessiner un <b>cylindre unitaire align sur l'axe z transform</b>  l'aide de transformation linaire.
   * Il faut pralablement tablir la couleur ou le matriel pour obtenir la couleur dsire.
   * 
   * @param gl Objet en rfrence au fonctionnalit de openGL.
   * @param scale La taille du cylindre.
   * @param rotation L'tat de rotation Rxyz du cylindre.
   * @param translation La position du cylindre.
   */
  public static void displayCylinder(final GL2 gl, SVector3d scale, SVector3d rotation, SVector3d translation)
  {
    // Dfinir les matrices de transformation de modle
    gl.glMatrixMode(GL2.GL_MODELVIEW);
    
    // Transformer le futur cube
    SGL.pushTransformationMatrix(gl, scale, rotation, translation);
    
    // Dessiner le cube unitaire
    displayCylinder(gl, SGLDisplay.DEFAULT_TUBE_DETAIL, SGLDisplay.DEFAULT_DISK_DETAIL);
    
    // Remettre les matrices de transformation d'origine
    gl.glPopMatrix();
  }
  
  /**
   * Mthode pour dessiner un <b>cone unitaire align sur l'axe z transform</b>  l'aide de transformation linaire.
   * Il faut pralablement tablir la couleur ou le matriel pour obtenir la couleur dsire.
   * 
   * @param gl Objet en rfrence au fonctionnalit de openGL.
   * @param scale La taille du cone.
   * @param rotation L'tat de rotation Rxyz du cone.
   * @param translation La position du cone.
   */
  public static void displayCone(final GL2 gl, SVector3d scale, SVector3d rotation, SVector3d translation)
  {
    // Dfinir les matrices de transformation de modle
    gl.glMatrixMode(GL2.GL_MODELVIEW);
    
    // Transformer le futur cube
    SGL.pushTransformationMatrix(gl, scale, rotation, translation);
    
    // Dessiner le cube unitaire
    displayCone(gl, SGLDisplay.DEFAULT_TUBE_DETAIL, SGLDisplay.DEFAULT_DISK_DETAIL);
    
    // Remettre les matrices de transformation d'origine
    gl.glPopMatrix();
  }
  
  /**
   * Mthode pour dessiner un cylindre unitaire align sur l'axe z dont l'une extrmit sera  l'origine.
   * 
   * @param gl
   * @param detail_tube
   * @param detail_disk
   */
  public static void displayCylinder(GL2 gl, int detail_tube, int detail_disk)
  {
    // Dessiner le tube.
    displayTube(gl, detail_tube);
    
    // Dessiner le disque du bas.
    displayDisk(gl, 1.0, false, detail_disk);
    
    // Dessiner le disque du haut.
    gl.glPushMatrix();
    
    gl.glTranslated(0.0, 0.0, 1.0);
    displayDisk(gl, 1.0, true, detail_disk);
    
    gl.glPopMatrix();
  }
  
  /**
   * ...
   * 
   * @param gl
   * @param detail
   */
  public static void displayCone(GL2 gl, int detail)
  {
    double dtheta = 2.0*Math.PI / detail;
    double du = 1.0 / detail;
    double  x0 = 0.0, y0 = 0.0,
            x1 = 0.0, y1 = 0.0,
            u0 = 0.0, u1 = 0.0,
            theta0 = 0.0, theta1 = 0.0;

    // Itration sur l'ensemble des rectangles qui vont dfinir le tube.
    for(int i = 0; i < detail; i++)
    {
      theta0 = theta1;
      u0 = u1;
      theta1 += dtheta;
      u1 += du;

      x0 = Math.cos(theta0);
      y0 = Math.sin(theta0);
      x1 = Math.cos(theta1);
      y1 = Math.sin(theta1);

      //  Dessiner un petit rectangle avec un section de la texture //
      gl.glBegin(GL2.GL_TRIANGLES);
        gl.glNormal3d(x0,y0,0.0);  gl.glTexCoord2d(u0, 1.0);  gl.glVertex3d(0.0, 0.0, 1.0);
        gl.glNormal3d(x0,y0,0.0);  gl.glTexCoord2d(u0, 0.0);  gl.glVertex3d(x0, y0, 0.0);
        gl.glNormal3d(x1,y1,0.0);  gl.glTexCoord2d(u1, 0.0);  gl.glVertex3d(x1, y1, 0.0);
      gl.glEnd();
    }
  }
  
  /**
   * Mthode pour dessiner un cone unitaire align sur l'axe z dont l'une extrmit sera  l'origine.
   * 
   * @param gl
   * @param detail_cone
   * @param detail_disk
   */
  public static void displayCone(GL2 gl, int detail_cone, int detail_disk)
  {
    // Dessiner le tube.
    displayCone(gl, detail_cone);
    
    // Dessiner le disque du bas.
    displayDisk(gl, 1.0, false, detail_disk);
  }
    
  /**
   * Mthode pour dessier en OpenGL une flche (forme d'un cylindre et d'un cne).
   * 
   * @param gl L'objet openGL.
   * @param color La couleur de la flche.
   * @param position La position de la flche (la queue)
   * @param orientation L'orientation de la flche
   * @param cylinder_length La longueur du cylindre.
   * @param cylinder_ray Le rayon du cylindre.
   * @param cone_length La longueur du cne.
   * @param cone_ray Le rayon du cne.
   * @param detail Le niveau de dtail des primitives.
   */
  public static void displayArrow(GL2 gl, SColor color, SVector3d position, SVector3d orientation, double cylinder_length, double cylinder_ray, double cone_length, double cone_ray, int detail)
  {
    // Initialiser la couleur du cube
    SGL.setColor(gl, color);
    
    // Dfinir les matrices de transformation de modle
    gl.glMatrixMode(GL2.GL_MODELVIEW);
    
    // Dfinir la rotation du tube et cne avec un quaternion.
    SQuaternion q = SQuaternion.toRotationQuaternion(SVector3d.UNITARY_Z, orientation);
    
    //----------------------
    // Dessiner le cylindre.
    //----------------------
    SGL.pushTransformationMatrix(gl, new SVector3d(cylinder_ray, cylinder_ray, cylinder_length), q, position);
    displayCylinder(gl, detail, detail);
    gl.glPopMatrix();
    
       
    //------------------
    // Dessiner le cne.
    //------------------
    
    // Prparation des matrices de transformation.
    gl.glPushMatrix();
            
    // Translation de la position finale.
    gl.glTranslated(position.getX(), position.getY(), position.getZ() ); 
    
    // Rotation. 
    SVector3d axis = q.rotationAxis();
    gl.glRotated(Math.toDegrees(q.rotationAngle()), axis.getX(), axis.getY(), axis.getZ()); 
    
    // Dplacer la base du cne  la point du cylindre.
    gl.glTranslated(0.0, 0.0, cylinder_length);   
    
    // Scale du cne.
    gl.glScaled(cone_ray, cone_ray, cone_length);  
    
    // Dessiner la primitive du cne (parallle  l'axe z).
    displayCone(gl, detail);
    
    // Retirer l'ensemble des matrices de transformation.
    gl.glPopMatrix();
  }
  
  /**
   * Mthode pour dessiner une collection de segment  partir d'une collection de position.
   * Un segment sera dessin pour l'ensemble des couples de positions.
   * 
   * @param gl
   * @param positions
   */
  public static void displayEdgesAllCombinations(GL2 gl, SVector3d[] positions)
  {
    gl.glBegin(GL2.GL_LINES);
    
    for(int i = 0; i < positions.length; i++)
      for(int j = i+1; j < positions.length; j++)
      {
        gl.glVertex3d(positions[i].getX(), positions[i].getY(), positions[i].getZ());
        gl.glVertex3d(positions[j].getX(), positions[j].getY(), positions[j].getZ());    
      }
    
    gl.glEnd();
    gl.glFlush();
  }
  
  /**
   * ...
   * 
   * @param gl
   * @param list
   */
  public static void displayLineStrip(GL2 gl, SVector3d[] positions)
  {
    gl.glBegin(GL2.GL_LINE_STRIP);
      
    for(int i = 0; i < positions.length; i++)
      gl.glVertex3d(positions[i].getX(), positions[i].getY(), positions[i].getZ());
    
    gl.glEnd();
    gl.glFlush();
  }
  
  /**
   * ...
   * 
   * @param gl
   * @param color
   * @param triangle
   */
  public static void displayTriangle(GL2 gl, SColor color, STriangle triangle)
  {
    displayTriangle(gl, color, triangle.getP1(), triangle.getP2(), triangle.getP3());
  }
  
  /**
   * ...
   * 
   * @param gl
   * @param p1
   * @param p2
   * @param p3
   */
  public static void displayTriangle(GL2 gl, SVector3d p1, SVector3d p2, SVector3d p3)
  {
    gl.glBegin(GL2.GL_TRIANGLES);
    
    gl.glVertex3d(p1.getX(), p1.getY(), p1.getZ());
    gl.glVertex3d(p2.getX(), p2.getY(), p2.getZ());
    gl.glVertex3d(p3.getX(), p3.getY(), p3.getZ());
    
    gl.glEnd();
    gl.glFlush();
  }
  
  /**
   * ...
   * 
   * @param gl
   * @param color
   * @param p1
   * @param p2
   * @param p3
   * @param n
   */
  public static void displayTriangle(GL2 gl, SColor color, SVector3d p1, SVector3d p2, SVector3d p3)
  {
    // Initialiser la couleur du cube
    SGL.setColor(gl, color);
    
    displayTriangle(gl, p1, p2, p3);
  }
  
  /**
   * ...
   * 
   * @param gl
   * @param p1
   * @param p2
   * @param p3
   * @param n
   */
  public static void displayTriangle(GL2 gl, SVector3d p1, SVector3d p2, SVector3d p3, SVector3d n)
  {
    gl.glBegin(GL2.GL_TRIANGLES);
    
    gl.glNormal3d(n.getX(), n.getY(), n.getZ()); 
    gl.glVertex3d(p1.getX(), p1.getY(), p1.getZ());
    
    gl.glNormal3d(n.getX(), n.getY(), n.getZ()); 
    gl.glVertex3d(p2.getX(), p2.getY(), p2.getZ());
    
    gl.glNormal3d(n.getX(), n.getY(), n.getZ()); 
    gl.glVertex3d(p3.getX(), p3.getY(), p3.getZ());
    
    gl.glEnd();
    gl.glFlush();
  }
  
  /**
   * ...
   * 
   * @param gl
   * @param p1
   * @param p2
   * @param p3
   * @param uv1
   * @param uv2
   * @param uv3
   * @param n1
   * @param n2
   * @param n3
   */
  public static void displayTriangle(GL2 gl, SVector3d p1, SVector3d p2, SVector3d p3, SVectorUV uv1, SVectorUV uv2, SVectorUV uv3, SVector3d n1, SVector3d n2, SVector3d n3)
  {
    gl.glBegin(GL2.GL_TRIANGLES);
    
    gl.glTexCoord2d(uv1.getU(), uv1.getV()); 
    gl.glNormal3d(n1.getX(), n1.getY(), n1.getZ()); 
    gl.glVertex3d(p1.getX(), p1.getY(), p1.getZ());
    
    gl.glTexCoord2d(uv2.getU(), uv2.getV()); 
    gl.glNormal3d(n2.getX(), n2.getY(), n2.getZ()); 
    gl.glVertex3d(p2.getX(), p2.getY(), p2.getZ());
    
    gl.glTexCoord2d(uv3.getU(), uv3.getV()); 
    gl.glNormal3d(n3.getX(), n3.getY(), n3.getZ()); 
    gl.glVertex3d(p3.getX(), p3.getY(), p3.getZ());
    
    gl.glEnd();
    gl.glFlush();
  }
  
  /**
   * ...
   * 
   * @param gl
   * @param p1
   * @param p2
   * @param p3
   * @param uv1
   * @param uv2
   * @param uv3
   * @param gl_texture_id
   */
  public static void displayTriangle(GL2 gl, SVector3d p1, SVector3d p2, SVector3d p3, SVectorUV uv1, SVectorUV uv2, SVectorUV uv3, int gl_texture_id)
  {
    // Activation des texture 2d.
    gl.glEnable(GL2.GL_TEXTURE_2D);
    
    // Accs  la texture #1.
    gl.glBindTexture(GL2.GL_TEXTURE_2D, gl_texture_id);   
    
    gl.glBegin(GL2.GL_TRIANGLES);
    
    gl.glTexCoord2d(uv1.getU(), uv1.getV()); gl.glVertex3d(p1.getX(), p1.getY(), p1.getZ());
    gl.glTexCoord2d(uv2.getU(), uv2.getV()); gl.glVertex3d(p2.getX(), p2.getY(), p2.getZ());
    gl.glTexCoord2d(uv3.getU(), uv3.getV()); gl.glVertex3d(p3.getX(), p3.getY(), p3.getZ());
    
    gl.glEnd();
    gl.glFlush();
  }
  
  /**
   * ...
   * 
   * @param gl
   * @param color
   * @param triangles
   */
  public static void displayTriangles(GL2 gl, SColor color, List<STriangle> triangles)
  {
    for(STriangle t : triangles)
      displayTriangle(gl, color, t);
  }
    
  /**
   * ...
   * 
   * @param gl
   * @param positions
   * @param scale
   * @param rotation
   * @param translation
   */
  public static void displayEdgesAllCombinations(GL2 gl, SVector3d[] positions, SVector3d scale, SVector3d rotation, SVector3d translation)
  {
    // Dfinir les matrices de transformation de modle
    gl.glMatrixMode(GL2.GL_MODELVIEW);
    
    // Transformer la futur sphre.
    SGL.pushTransformationMatrix(gl, scale, rotation, translation);
    
    // Dessiner le cube unitaire
    displayEdgesAllCombinations(gl, positions);
    
    // Remettre les matrices de transformation d'origine
    gl.glPopMatrix();
  }
  
  /**
   * ...
   * @param gl
   * @param list
   * @param scale
   * @param rotation
   * @param translation
   */
  public static void displayLineStrip(GL2 gl, SVector3d[] positions, SVector3d scale, SVector3d rotation, SVector3d translation)
  {
    // Dfinir les matrices de transformation de modle
    gl.glMatrixMode(GL2.GL_MODELVIEW);
    
    // Transformer la futur sphre.
    SGL.pushTransformationMatrix(gl, scale, rotation, translation);
    
    // Dessiner le cube unitaire
    displayLineStrip(gl, positions);
    
    // Remettre les matrices de transformation d'origine
    gl.glPopMatrix();
  }
      
  /**
   * ...
   * 
   * @param gl
   * @param color
   * @param positions
   * @param scale
   * @param rotation
   * @param translation
   */
  public static void displayEdgesAllCombinations(GL2 gl, SColor color, SVector3d[] positions, SVector3d scale, SVector3d rotation, SVector3d translation)
  {
    // Initialiser la couleur du cube
    SGL.setColor(gl, color);
    
    // Dessiner avec les transformations appropries
    displayEdgesAllCombinations(gl, positions, scale, rotation, translation);
  }
  
  /**
   * ...
   * 
   * @param gl
   * @param color
   * @param list
   * @param scale
   * @param rotation
   * @param translation
   */
  public static void displayLineStrip(GL2 gl, SColor color, SVector3d[] positions, SVector3d scale, SVector3d rotation, SVector3d translation)
  {
    // Initialiser la couleur du cube
    SGL.setColor(gl, color);
    
    // Dessiner avec les transformations appropries
    displayLineStrip(gl, positions, scale, rotation, translation);
  }
     
  /**
   * Mthode pour dessiner un vecteur  l'aide d'une flche.
   * 
   * @param gl L'objet OpenGL.
   * @param position La position de la queue de la flche.
   * @param color La couleur de la flche.
   * @param max_vector_modulus Le module maximal du vecteur.
   * @param division Le nombre de sous-division du module maximal du vecteur.
   * @param max_arrow_size La longueur maximale de la flche.
   * @param vector Le vecteur  dessiner.
   * @throws IllegalArgumentException S'il y a un paramtre illgal.
   */
  public static void displayVector(GL2 gl, SVector3d position, SColor color, double max_vector_modulus, int division, double max_arrow_size, SVector3d vector) throws IllegalArgumentException
  {
    displayVector(gl, position, color, SColor.YELLOW, SColor.ORANGE, max_vector_modulus, division, max_arrow_size, vector);
  }
  
  
  /**
   * Mthode pour dessiner un vecteur  l'aide d'une flche.
   * 
   * @param gl L'objet OpenGL.
   * @param position La position de la queue de la flche
   * @param color La couleur du vecteur.
   * @param color_saturation La couleur de saturation.
   * @param color_minimal La couleur de taille minimale.
   * @param max_vector_modulus Le module maximal du vecteur.
   * @param division Le nombre de sous-division du module maximal du vecteur.
   * @param max_arrow_size La longueur maximale de la flche.
   * @param vector Le vecteur  dessiner.
   * @throws IllegalArgumentException S'il y a un paramtre illgal.
   */
  public static void displayVector(GL2 gl, SVector3d position, SColor color, SColor color_saturation, SColor color_minimal, double max_vector_modulus, int division, double max_arrow_size, SVector3d vector) throws IllegalArgumentException
  {
    if(max_vector_modulus <= 0.0)
      throw new IllegalArgumentException("Erreur SGLDisplay 001 : Le module unitaire de rfrence m = " + max_vector_modulus + " doit tre suprieur  0.");
       
    double nb_unit_length = vector.modulus() / max_vector_modulus;
    
    double division_length = max_arrow_size / (double)division;
    
    // Dessiner la flche dsigant le champ s'il n'est pas de longueur zro.
    if(nb_unit_length  != 0.0)
    {
      // Nous aurons deux versions de flche :
      // 1) flche trop longue (jaune)
      // 2) trop courte (orange)
      // 3) flche normale (couleur normale)
      
      double cylinder_length;
      SColor c;
      
      if(nb_unit_length > 1) // trop longue
      {
        cylinder_length =  max_arrow_size-division_length*0.7;
        c = color_saturation;
      }
      else
        if(nb_unit_length < 1.0/(double)division) // trop courte
        {
          cylinder_length = division_length - division_length*0.7;
          c = color_minimal;
        }
        else
        {
          cylinder_length = nb_unit_length*max_arrow_size - division_length*0.7;
          c = color;
        }
      
      // Dessiner la flche.
      displayArrow(gl, c, position, vector.normalize(), cylinder_length, division_length*0.3, division_length*0.7, division_length*0.6, DEFAULT_CONE_DETAIL);
    }
  }
  
}
