/**
 * 
 */
package sim.physics;

import java.util.List;

import sim.math.SVector3d;

/**
 * La classe <b>SGravitationnalParticleSystem</b> reprsente un systme de particules o il y a application de la force gravitationnelle entre les particules.
 * 
 * @author Simon Vzina
 * @since 2023-09-13
 * @version 2023-10-24
 */
public class SGravitationnalParticleSystem extends SParticleSystem {

  /**
   * Construteur d'un systme de particules gravitationnelles  partir d'un tableau de particules.
   * 
   * @param particles Les particules du systme.
   */
  public SGravitationnalParticleSystem(SParticle... particles) 
  {
    super(particles);
  }

  /**
   * Construteur d'un systme de particules gravitationnelles  partir d'une liste de particules.
   * 
   * @param particles Les particules du systme.
   */
  public SGravitationnalParticleSystem(List<SParticle> list)
  {
    super(list);
  }

  @Override
  public SVector3d interactionForce(SParticle p_source, SParticle p_target) throws IllegalArgumentException
  {
    // Vrification que les deux particules ne sont pas identiques.
    if(p_source.asSameID(p_target))
      throw new IllegalArgumentException("Erreur SGravitationnalParticleSystem 001 : L'interaction est indtermine, car on ne peut pas faire l'interaction encore deux particules de mme numro d'identification.");
       
    return SGravitation.gravitationLaw(p_source.getMass(), p_source.getPosition(), p_target.getMass(), p_target.getPosition());
  }

  @Override
  public double interactionEnergy(SParticle p1, SParticle p2) throws IllegalArgumentException 
  {
    // Vrification que les deux particules ne sont pas identiques.
    if(p1.asSameID(p2))
      throw new IllegalArgumentException("Erreur SGravitationnalParticleSystem 002 : L'interaction est indtermine, car on ne peut pas faire l'interaction encore deux particules de mme numro d'identification.");
    
    return SGravitation.gravitationalEnergy(p1.getMass(), p1.getPosition(), p2.getMass(), p2.getPosition());
  }
    
}
