package sim.physics;

import java.util.Objects;

import sim.exception.SConstructorException;
import sim.math.SVector3d;

/**
 * La classe <b>SKineticComponent</b> reprsente des composants cintique regroupant la position et la vitesse d'un objet.
 * 
 * @author Simon Vezina
 * @since 2020-05-29
 * @version 2023-10-24
 */
public class SKineticComponent {
   
  /**
   * Le paramtre <b>position</b> reprsente la position.
   */
  private final SVector3d position;
  
  /**
   * Le paramtre <b>speed</b> reprsente la vitesse.
   */
  private final SVector3d speed;
  
  /**
   * Le constructeur d'un composant cintique comprenant une position et une vitesse.
   * 
   * @param position La position.
   * @param speed La vitesse.
   * @throws SConstructorException Si l'un des composants (position ou vitesse) est null.
   */
  public SKineticComponent(SVector3d position, SVector3d speed) throws SConstructorException
  {
    if(position == null)
      throw new SConstructorException("Erreur SKineticComponent 001 : La position ne peut pas tre null.");
    
    if(speed == null)
      throw new SConstructorException("Erreur SKineticComponent 002 : La vitesse ne peut pas tre null.");
    
    this.position = position;
    this.speed = speed;
  }
    
  /**
   * Mthode pour obtenir la position.
   * 
   * @return La position.
   */
  public SVector3d getPosition() 
  {
    return this.position;
  }

  /**
   * Mthode pour obtenir la vitesse.
   * 
   * @return La vitesse.
   */
  public SVector3d getSpeed() 
  {
    return this.speed;
  }

  

  @Override
  public String toString() 
  {
    return "SKineticComponent [position=" + position + ", speed=" + speed + "]";
  }

  @Override
  public int hashCode() 
  {
    return Objects.hash(position, speed);
  }

  @Override
  public boolean equals(Object obj) 
  {
    if (this == obj)
      return true;
    
    if (obj == null)
      return false;
    
    if (getClass() != obj.getClass())
      return false;
    
    SKineticComponent other = (SKineticComponent) obj;
    return Objects.equals(position, other.position) && Objects.equals(speed, other.speed);
  }
  
}
