/**
 * 
 */
package sim.physics;

import sim.exception.SConstructorException;
import sim.math.SVector3d;

/**
 * La classe <b>SPlanet</b> reprsente une plante o la force gravitationnelle peut tre applique.
 * 
 * @author Simon Vzina
 * @since 2023-09-27
 * @version 2023-10-11
 */
public class SPlanet {

  /**
   * La variable <b>particle</b> reprsente la particule correspondant  la plante.
   */
  protected SParticle particle;
  
  /**
   * La variable <b>ray</b> reprsente le rayon de la plante.
   */
  protected final double ray;
  
  /**
   * La variable <b>texture_name</b> reprsente la texture correspondant  la plante.
   */
  protected final String texture_name;
  
  /**
   * ...
   * 
   * @param mass
   * @param ray
   * @param position
   * @param speed
   * @param texture_name
   * @throws SConstructorException
   */
  public SPlanet(double mass, double ray, SVector3d position, SVector3d speed, String texture_name) throws SConstructorException 
  {
    if(ray < 0.0)
      throw new SConstructorException("Erreur SPlanet 001 : Le rayon d'une plante ne peut pas tre ngatif.");
    
    this.particle = new SParticle(mass, position, speed);
    
    this.ray = ray;
    this.texture_name = texture_name;
  }

  /**
   * Mthode pour gnrer une copie de la plante courant. Il est  not qu'une nouvelle particule sera dfinie pour cette plante (nouveau numro ID) pour l'identifier.
   * Ainsi, cette mthode n'est pas identique  la mthode <i>clone</i>.
   */
  public SPlanet copy()
  {
    return new SPlanet(this.particle.getMass(), this.ray, this.particle.getPosition(), this.particle.getSpeed(), this.texture_name);
  }
  
  /**
   * ...
   * 
   * @return
   */
  public SVector3d getPosition()
  {
    return this.particle.getPosition();
  }
  
  /**
   * Mthode pour obtenir la particule dfinissant la plante.
   * 
   * @return La reprsentation en particule de la plante.
   */
  public SParticle getParticle()
  {
    return this.particle;
  }
  
  /**
   * ...
   * 
   * @return
   */
  public double getRay()
  {
    return ray;
  }
  
  /**
   * ...
   * 
   * @return
   */
  public String getTextureName()
  {
    return this.texture_name;
  }

  @Override
  public String toString() 
  {
    return "SPlanet [particle=" + particle + ", ray=" + ray + ", texture_name=" + texture_name + "]";
  }
  
}
