/**
 * 
 */
package sim.readwrite.object;

import java.io.BufferedWriter;
import java.io.IOException;

import sim.readwrite.SAbstractRW;
import sim.util.SBufferedReader;
import sim.util.SInitializationException;
import sim.util.SKeyWordDecoder;
import sim.util.SReadingException;
import sim.util.SStringUtil;

/**
 * La classe <b>SDisplayParameterRW</b> reprsente un configurateur d'affichage pouvant tre lu dans un fichier texte.
 * 
 * @author Simon Vzina
 * @since 2023-10-24
 * @version 2023-10-25
 */
public class SDisplayParameterRW extends SAbstractRW {

  /**
   * La constante <b>KEYWORD_PARAMETER</b> correspond  un tableau contenant l'ensemble des mots cls 
   *  utiliser reconnus lors de la dfinition de l'objet par une lecture en fichier.
   */
  private static final String[] KEYWORD_PARAMETER = {
   SKeyWordDecoder.KW_NB_FADING, SKeyWordDecoder.KW_FADING_INTERVAL, SKeyWordDecoder.KW_RESCALING_FACTOR
  };
    
  /**
   * ...
   */
  private static final int DEFAULT_NB_FADING = 0;
  
  /**
   * ...
   */
  private static final int DEFAULT_FADING_INTERVAL = 1;
  
  /**
   * ...
   */
  private static final double DEFAULT_RESCALING_FACTOR = 1.0;
  
  /**
   * ...
   */
  private int nb_fading;
  
  /**
   * ...
   */
  private int fading_interval;
  
  /**
   * ...
   */
  private double rescaling_factor;
  
  /**
   * 
   */
  public SDisplayParameterRW() 
  {
    this.nb_fading = DEFAULT_NB_FADING;
    this.fading_interval = DEFAULT_FADING_INTERVAL;
    this.rescaling_factor = DEFAULT_RESCALING_FACTOR;
  }

  /**
   * ...
   * 
   * @param sbr
   * @throws SReadingExceltion
   */
  public SDisplayParameterRW(SBufferedReader sbr) throws SReadingException
  {
    this();   
    
    try{
      read(sbr);
    }catch(SInitializationException e){
      throw new SReadingException("Erreur SDisplayParameterRW 001 : Une erreur d'initialisation est survenue.", e);
    }
  }
  
  /**
   * ...
   * 
   * @return
   */
  public int getNbFading()
  {
    return this.nb_fading;
  }
  
  /**
   * ...
   * 
   * @return
   */
  public int getFadingInterval()
  {
    return this.fading_interval;
  }
  
  /**
   * ...
   * 
   * @return
   */
  public double getRescalingFactor()
  {
    return this.rescaling_factor;
  }
  
  @Override
  public String getRWName() 
  {
    return SKeyWordDecoder.KW_DISPLAY_PARAMETER;
  }

  @Override
  public void writeInformation(BufferedWriter bw) throws IOException
  {
    super.writeInformation(bw);
    
    bw.write(SKeyWordDecoder.KW_RESCALING_FACTOR);
    bw.write("\t");
    bw.write(Double.toString(this.rescaling_factor));
    bw.write(" real unit / display unit");
    bw.write(SStringUtil.END_LINE_CARACTER);
    
    bw.write(SKeyWordDecoder.KW_NB_FADING);
    bw.write("\t\t");
    bw.write(Integer.toString(this.nb_fading));
    bw.write(" images ");
    bw.write(SStringUtil.END_LINE_CARACTER);
    
    bw.write(SKeyWordDecoder.KW_FADING_INTERVAL);
    bw.write("\t\t");
    bw.write(Integer.toString(this.fading_interval));
    bw.write(" images");
    bw.write(SStringUtil.END_LINE_CARACTER);
  }
   
  @Override
  protected void initialization() throws SInitializationException
  {
    
  }
  
  @Override
  protected boolean read(SBufferedReader sbr, int code, String remaining_line) throws SReadingException
  {
    switch(code)
    {
      case SKeyWordDecoder.CODE_RESCALING_FACTOR :  this.rescaling_factor = this.readDoubleGreaterThanZero(remaining_line, SKeyWordDecoder.KW_RESCALING_FACTOR); return true;
     
      case SKeyWordDecoder.CODE_NB_FADING :         this.nb_fading = this.readIntEqualOrGreaterThanZero(remaining_line, SKeyWordDecoder.KW_NB_FADING); return true;
      
      case SKeyWordDecoder.CODE_FADING_INTERVAL :   this.fading_interval = this.readIntGreaterThanZero(remaining_line, SKeyWordDecoder.KW_FADING_INTERVAL); return true;
      
      default : return super.read(sbr, code, remaining_line);
    } 
  }
  
  @Override
  public String[] getRWParameterName()
  {
    String[] other_parameters = super.getRWParameterName();
    
    return SStringUtil.merge(other_parameters, KEYWORD_PARAMETER);
  }
  

}
