/**
 * 
 */
package sim.readwrite.object;

import java.io.BufferedWriter;
import java.io.IOException;

import sim.readwrite.SAbstractRW;
import sim.util.SBufferedReader;
import sim.util.SInitializationException;
import sim.util.SKeyWordDecoder;
import sim.util.SReadingException;
import sim.util.SStringUtil;

/**
 * La classe <b>SMouseSetupRW</b> permet la lecture et l'criture dans un fichier visant  configurer le fonctionnement d'une sourie de contrle.
 * 
 * @author Simon Vzina
 * @version 2023-09-27
 * @version 2023-10-07
 */
public class SMouseSetupRW extends SAbstractRW {

  /**
   * La constante <b>KEYWORD_PARAMETER</b> correspond  un tableau contenant l'ensemble des mots cls 
   *  utiliser reconnus lors de la dfinition de l'objet par une lecture en fichier.
   */
  private static final String[] KEYWORD_PARAMETER = {
    
    // Pour les scnes de ray tracer.  
    SKeyWordDecoder.KW_MOVEMENT_SENSIBILITY,       
    SKeyWordDecoder.KW_DELTA_MOVEMENT,      
    SKeyWordDecoder.KW_ROTATION_SENSIBILITY,       
    SKeyWordDecoder.KW_DELTA_ROTATION             
  };
  
  private static final double DEFAULT_MOVEMENT_SENSIBILITY = 0.1;
  private static final double DEFAULT_DELTA_MOVEMENT = 1.0;
  private static final double DEFAULT_ROTATION_SENSIBILITY = 0.1;
  private static final double DEFAULT_DELTA_ROTATION = 1.0;
  
  /**
   * La constante <b>MAX_DELTA_ROTATION</b> reprsente la variation maximale d'angle lors de la rotation d'une camra gal  {@value} en dgrs.
   */
  private static final double MAX_DELTA_ROTATION = 10.0;
  
  protected double movement_sensibility;
  protected double delta_movement;
  protected double rotation_sensibility;
  protected double delta_rotation;
  
  /**
   * 
   */
  public SMouseSetupRW() 
  {
    this.movement_sensibility = DEFAULT_MOVEMENT_SENSIBILITY;
    this.delta_movement = DEFAULT_DELTA_MOVEMENT;
    this.rotation_sensibility = DEFAULT_ROTATION_SENSIBILITY;
    this.delta_rotation = DEFAULT_DELTA_ROTATION;
  }

  /**
   * ...
   * 
   * @param sbr
   * @throws SReadingException
   */
  public SMouseSetupRW(SBufferedReader sbr) throws SReadingException
  {
    this();   
    
    try{
      read(sbr);
    }catch(SInitializationException e){
      throw new SReadingException("Erreur SMouseSetupRW 001 : Une erreur d'initialisation est survenue.", e);
    }
  }
  
  @Override
  public String getRWName()
  {
    return SKeyWordDecoder.KW_MOUSE_SETUP;
  }
  
  /**
   * ...
   * 
   * @return
   */
  public double getMovementSensibility()
  {
    return this.movement_sensibility;
  }
  
  /**
   * ...
   * 
   * @return
   */
  public double getDeltaMovement()
  {
    return this.delta_movement;
  }
  
  /**
   * ...
   * 
   * @return
   */
  public double getRotationSensibility()
  {
    return this.rotation_sensibility;
  }
  
  /**
   * ...
   * 
   * @return
   */
  public double getDeltaRotation()
  {
    return this.delta_rotation;
  }
  
  @Override
  public void writeInformation(BufferedWriter bw) throws IOException
  {
    super.writeInformation(bw);
    
    bw.write(SKeyWordDecoder.KW_MOVEMENT_SENSIBILITY);
    bw.write("\t\t");
    bw.write(Double.toString(this.movement_sensibility));
    bw.write(" (entre 0.0 et 1.0)");
    bw.write(SStringUtil.END_LINE_CARACTER);
    
    bw.write(SKeyWordDecoder.KW_DELTA_MOVEMENT);
    bw.write("\t\t\t");
    bw.write(Double.toString(this.delta_movement));
    bw.write(" m");
    bw.write(SStringUtil.END_LINE_CARACTER);
    
    bw.write(SKeyWordDecoder.KW_ROTATION_SENSIBILITY);
    bw.write("\t\t");
    bw.write(Double.toString(this.rotation_sensibility));
    bw.write(" (entre 0.0 et 1.0)");
    bw.write(SStringUtil.END_LINE_CARACTER);
    
    bw.write(SKeyWordDecoder.KW_DELTA_ROTATION);
    bw.write("\t\t\t");
    bw.write(Double.toString(this.delta_rotation));
    bw.write(" degrs");
    bw.write(SStringUtil.END_LINE_CARACTER);
  }
  
  @Override
  protected void initialization() throws SInitializationException
  {
    // rien  faire.
  }
  
  @Override
  protected boolean read(SBufferedReader sbr, int code, String remaining_line) throws SReadingException
  {
    switch(code)
    {
      case SKeyWordDecoder.CODE_MOVEMENT_SENSIBILITY : this.movement_sensibility = this.readDoubleEqualOrGreaterThanValueAndEqualOrSmallerThanValue(remaining_line, 0.0, 1.0, SKeyWordDecoder.KW_MOVEMENT_SENSIBILITY); return true;
      
      case SKeyWordDecoder.CODE_DELTA_MOVEMENT : this.delta_movement = this.readDoubleGreaterThanZero(remaining_line, SKeyWordDecoder.KW_DELTA_MOVEMENT); return true;
      
      case SKeyWordDecoder.CODE_ROTATION_SENSIBILITY : this.rotation_sensibility = this.readDoubleEqualOrGreaterThanValueAndEqualOrSmallerThanValue(remaining_line, 0.0, 1.0, SKeyWordDecoder.KW_ROTATION_SENSIBILITY); return true;
      
      case SKeyWordDecoder.CODE_DELTA_ROTATION : this.delta_rotation = this.readDoubleEqualOrGreaterThanValueAndEqualOrSmallerThanValue(remaining_line, 0.0, MAX_DELTA_ROTATION, SKeyWordDecoder.KW_DELTA_ROTATION); return true;
      
      default : return super.read(sbr, code, remaining_line);
    } 
  }
  
  @Override
  public String[] getRWParameterName()
  {
    String[] other_parameters = super.getRWParameterName();
    
    return SStringUtil.merge(other_parameters, KEYWORD_PARAMETER);
  }
  
}
