/**
 * 
 */
package sim.readwrite.object;

import java.io.BufferedWriter;
import java.io.IOException;

import sim.math.SVector3d;
import sim.physics.SPlanet;
import sim.readwrite.SAbstractRW;
import sim.util.SBufferedReader;
import sim.util.SInitializationException;
import sim.util.SKeyWordDecoder;
import sim.util.SReadingException;
import sim.util.SStringUtil;

/**
 * La classe <b>SPlanetRW</b> reprsente une plante initialise lors d'une lecture en fichier et pouvant tre crite dans un fichier texte.
 * 
 * @author Simon Vzina
 * @version 2023-09-27
 * @version 2023-09-27
 */
public class SPlanetRW extends SAbstractRW {

  /**
   * La constante <b>KEYWORD_PARAMETER</b> correspond  un tableau contenant l'ensemble des mots cls 
   *  utiliser reconnus lors de la dfinition de l'objet par une lecture en fichier.
   */
  private static final String[] KEYWORD_PARAMETER = {
    SKeyWordDecoder.KW_POSITION,
    SKeyWordDecoder.KW_SPEED, 
    SKeyWordDecoder.KW_MASS,
    SKeyWordDecoder.KW_RAY,
    SKeyWordDecoder.KW_TEXTURE_MATERIAL
  };
  
  /**
   * La constante <b>DEFAULT_POSITION</b> correspond  la position de la plante par dfaut.
   */
  private static final SVector3d DEFAULT_POSITION = new SVector3d(0.0, 0.0, 0.0);
  
  /**
   * ...
   */
  private static final SVector3d DEFAULT_SPEED = new SVector3d(0.0, 0.0, 0.0);
  
  /**
   * ...
   */
  private static final double DEFAULT_MASS = 1e3;
  
  /**
   * ...
   */
  private static final double DEFAULT_RAY = 10.0;
  
  /**
   * ...
   */
  private static final String DEFAULT_TEXTURE_MATERIAL = "no_texture_name";
 
  protected SVector3d position;
  protected SVector3d speed;
  protected double mass;
  protected double ray;
  protected String texture_name;
  
  /**
   * 
   */
  public SPlanetRW() 
  {
    super();
    
    this.position = DEFAULT_POSITION;
    this.speed = DEFAULT_SPEED;
    this.mass = DEFAULT_MASS;
    this.ray = DEFAULT_RAY;
    this.texture_name = DEFAULT_TEXTURE_MATERIAL;
  }

  /**
   * ...
   * 
   * @param sbr
   * @throws SReadingException
   */
  public SPlanetRW(SBufferedReader sbr) throws SReadingException
  {
    this();   
    
    try{
      read(sbr);
    }catch(SInitializationException e){
      throw new SReadingException("Erreur SPlanetRW 001 : Une erreur d'initialisation est survenue.", e);
    }
  }
  
  /**
   * ...
   * 
   * @return
   */
  public SPlanet toSPlanet()
  {
    return new SPlanet(this.mass, this.ray, this.position, this.speed, this.texture_name);
  }
  
  @Override
  public String getRWName()
  {
    return SKeyWordDecoder.KW_PLANET;
  }

  @Override
  public void writeInformation(BufferedWriter bw) throws IOException
  {
    super.writeInformation(bw);
    
    bw.write(SKeyWordDecoder.KW_POSITION);
    bw.write("\t\t");
    this.position.write(bw);
    bw.write(SStringUtil.END_LINE_CARACTER);
    
    bw.write(SKeyWordDecoder.KW_SPEED);
    bw.write("\t\t\t");
    this.speed.write(bw);
    bw.write(SStringUtil.END_LINE_CARACTER);
    
    bw.write(SKeyWordDecoder.KW_MASS);
    bw.write("\t\t\t");
    bw.write(Double.toString(this.mass));
    bw.write(" kg");
    bw.write(SStringUtil.END_LINE_CARACTER);
    
    bw.write(SKeyWordDecoder.KW_RAY);
    bw.write("\t\t\t");
    bw.write(Double.toString(this.ray));
    bw.write(" m");
    bw.write(SStringUtil.END_LINE_CARACTER);
    
    bw.write(SKeyWordDecoder.KW_TEXTURE_MATERIAL);
    bw.write("\t");
    bw.write(this.texture_name);
    bw.write(SStringUtil.END_LINE_CARACTER);
  }
  
  @Override
  protected void initialization() throws SInitializationException
  {
    // rien  faire.
  }
  
  @Override
  protected boolean read(SBufferedReader sbr, int code, String remaining_line) throws SReadingException
  {
    switch(code)
    {
      case SKeyWordDecoder.CODE_POSITION : this.position = this.readSVector3d(remaining_line, SKeyWordDecoder.KW_POSITION); return true;
      
      case SKeyWordDecoder.CODE_SPEED : this.speed = this.readSVector3d(remaining_line, SKeyWordDecoder.KW_SPEED); return true;
       
      case SKeyWordDecoder.CODE_MASS : this.mass = this.readDoubleGreaterThanZero(remaining_line, SKeyWordDecoder.KW_MASS); return true;
      
      case SKeyWordDecoder.CODE_RAY : this.ray = this.readDoubleGreaterThanZero(remaining_line, SKeyWordDecoder.KW_RAY); return true;
      
      case SKeyWordDecoder.CODE_TEXTURE_MATERIAL : this.texture_name = this.readStringNotEmpty(remaining_line, SKeyWordDecoder.KW_TEXTURE_MATERIAL); return true;
      
      default : return super.read(sbr, code, remaining_line);
    } 
  }
  
  @Override
  public String[] getRWParameterName()
  {
    String[] other_parameters = super.getRWParameterName();
    
    return SStringUtil.merge(other_parameters, KEYWORD_PARAMETER);
  }
  
}
