/**
 * 
 */
package sim.readwrite.object;

import java.io.BufferedWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import sim.exception.SConstructorException;
import sim.readwrite.SAbstractRW;
import sim.util.SBufferedReader;
import sim.util.SKeyWordDecoder;
import sim.util.SReadingException;
import sim.util.SStringUtil;

/**
 * La classe <b>SSceneRW</b> reprsente un objet scne pouvant 
 * lire ses informations et crire ses informations dans un fichier texte.
 * 
 * @author Simon Vezina
 * @since 2017-11-16
 * @version 2023-10-24 (Version : Les orbites (2023-10-30)
 */
public class SSceneRW extends SAbstractRW {

  /**
   * La constante <b>KEYWORD_PARAMETER</b> correspond  un tableau contenant l'ensemble des mots cls 
   *  utiliser reconnus lors de la dfinition de l'objet par une lecture en fichier.
   */
  private static final String[] KEYWORD_PARAMETER = {
    
    // Pour les scnes de ray tracer.  
    SKeyWordDecoder.KW_CAMERA, SKeyWordDecoder.KW_VIEWPORT, SKeyWordDecoder.KW_RAYTRACER,
    SKeyWordDecoder.KW_PRIMITIVE, SKeyWordDecoder.KW_BLINN_MATERIAL, SKeyWordDecoder.KW_TEXTURE_MATERIAL, 
    SKeyWordDecoder.KW_AMBIENT_LIGHT, SKeyWordDecoder.KW_DIRECTIONAL_LIGHT, SKeyWordDecoder.KW_POINT_LIGHT,
    SKeyWordDecoder.KW_LINEAR_APERTURE_LIGHT, SKeyWordDecoder.KW_RECTANGULAR_APERTURE_LIGHT,
    SKeyWordDecoder.KW_ELLIPTICAL_APERTURE_LIGHT,SKeyWordDecoder.KW_APERTURE_MASK_LIGHT,
    SKeyWordDecoder.KW_MODEL, 
        
    // Pour les autres types de scne.
    SKeyWordDecoder.KW_PLANET, SKeyWordDecoder.KW_STAR,
    SKeyWordDecoder.KW_SIMULATION_PARAMETER, SKeyWordDecoder.KW_DISPLAY_PARAMETER, SKeyWordDecoder.KW_MOUSE_SETUP 
  };
  
  
  /**
   * La variable <b>camera_list</b> reprsente la liste des cameras de la scne.
   */
  private final List<SCameraRW> camera_list;
  
  /**
   * La variable <b>planet_list</b> reprsente une liste des plantes de la scne.
   */
  private final List<SPlanetRW> planet_list;
  
  /**
   * La variable <b>star_list</b> reprsente une liste des toiles de la scne.
   */
  private final List<SStarRW> star_list;
  
  /**
   * ... 
   */
  private SSimulationParameterRW simulation_parameter;
  
  /**
   * ... 
   */
  private SDisplayParameterRW display_parameter;
  
  /**
   * 
   */
  private SMouseSetupRW mouse_setup;
  
  //---------------------
  // LES CONSTRUCTEURS //
  //---------------------
   
  /**
   * Constructeur d'un lecteur de scne  partir du nom du fichier en lecteur pour dfinir la scne.
   * 
   * @param file_name Le nom du fichier.
   * @throws SConstructorException Si le nom du fichier est null.
   */
  public SSceneRW(String file_name) throws SConstructorException
  {
    super(file_name);
    
    if(file_name == null)
      throw new SConstructorException("Erreur SSceneRW 001 : Un scne ne peut pas tre dfini avec un nom de fichier null.");
    
    this.camera_list = new ArrayList<SCameraRW>();  
    this.planet_list = new ArrayList<SPlanetRW>();
    this.star_list = new ArrayList<SStarRW>();
    this.simulation_parameter = new SSimulationParameterRW();
    this.display_parameter = new SDisplayParameterRW();
    this.mouse_setup = new SMouseSetupRW();
  }
  
  /**
   * ...
   * 
   * @return
   */
  public List<SCameraRW> getCameraList()
  {
    return this.camera_list;
  }
  
    
  /**
   * ...
   * 
   * @return
   */
  public List<SPlanetRW> getPlanetList()
  {
    return this.planet_list;
  }
  
  /**
   * ...
   * 
   * @return
   */
  public List<SStarRW> getStarList()
  {
    return this.star_list;
  }
  
  
  /**
   * ...
   * 
   * @return
   */
  public SSimulationParameterRW getSimulationParameter()
  {
    return this.simulation_parameter;
  }
  
  /**
   * ...
   * 
   * @return
   */
  public SDisplayParameterRW getDisplayParameter()
  {
    return this.display_parameter;
  }
  
  /**
   * ...
   * 
   * @return
   */
  public SMouseSetupRW getMouseSetup()
  {
    return this.mouse_setup;
  }
  
  @Override
  protected boolean read(SBufferedReader sbr, int code, String remaining_line)throws SReadingException
  {
      switch(code)
      {
        case SKeyWordDecoder.CODE_CAMERA :                this.camera_list.add(new SCameraRW(sbr)); return true;
               
        case SKeyWordDecoder.CODE_PLANET :                this.planet_list.add(new SPlanetRW(sbr)); return true;
        case SKeyWordDecoder.CODE_STAR :                  this.star_list.add(new SStarRW(sbr)); return true;
        
        case SKeyWordDecoder.CODE_SIMULATION_PARAMETER :  this.simulation_parameter = new SSimulationParameterRW(sbr); return true;
        case SKeyWordDecoder.CODE_DISPLAY_PARAMETER :     this.display_parameter = new SDisplayParameterRW(sbr); return true;
        
        case SKeyWordDecoder.CODE_MOUSE_SETUP :           this.mouse_setup = new SMouseSetupRW(sbr); return true;
        
        /*
        case SKeyWordDecoder.CODE_VIEWPORT :              viewport = new SViewport(sbr); return true;
                        
        case SKeyWordDecoder.CODE_RAYTRACER :             raytracer_builder = new SRaytracerBuilder(sbr); return true;
        
        
                                
        case SKeyWordDecoder.CODE_BLINN_MATERIAL:         material_list.add(new SBlinnMaterial(sbr)); return true;
        
        case SKeyWordDecoder.CODE_TEXTURE_MATERIAL :      material_list.add(new SBlinnTextureMaterial(sbr)); return true;
        
        case SKeyWordDecoder.CODE_AMBIENT_LIGHT :         light_list.add(new SAmbientLight(sbr)); return true;
                                
        case SKeyWordDecoder.CODE_DIRECTIONAL_LIGHT :     light_list.add(new SDirectionalLight(sbr)); return true;
                                
        case SKeyWordDecoder.CODE_POINT_LIGHT :           light_list.add(new SPointLight(sbr)); return true;
        
        case SKeyWordDecoder.CODE_LINEAR_APERTURE_LIGHT : light_list.add(new SLinearApertureLight(sbr)); return true;
        
        case SKeyWordDecoder.CODE_RECTANGULAR_APERTURE_LIGHT : light_list.add(new SRectangularApertureLight(sbr)); return true;
        
        case SKeyWordDecoder.CODE_ELLIPTICAL_APERTURE_LIGHT : light_list.add(new SEllipticalApertureLight(sbr)); return true;
        
        case SKeyWordDecoder.CODE_APERTURE_MASK_LIGHT : light_list.add(new SApertureMaskLight(sbr)); return true;
        
        
        case SKeyWordDecoder.CODE_MODEL :   SModelReader m_reader = new SModelReader(sbr);
        
                                            if(m_reader.asRead())
                                              model_list.add((SModel)m_reader.getValue());
                                            
                                            return true;
        */
        
        default : return super.read(sbr, code, remaining_line);
      }
  }

  @Override
  public void writeInformation(BufferedWriter bw) throws IOException
  {
    super.writeInformation(bw);
  
    simulation_parameter.write(bw);
    bw.write(SStringUtil.END_LINE_CARACTER);
  
    display_parameter.write(bw);
    bw.write(SStringUtil.END_LINE_CARACTER);
    
    mouse_setup.write(bw);
    bw.write(SStringUtil.END_LINE_CARACTER);
        
    if(!camera_list.isEmpty())
    {
      for(SCameraRW c : camera_list)
        c.write(bw);
      bw.write(SStringUtil.END_LINE_CARACTER);
    }
        
    if(!planet_list.isEmpty())
    {
      for(SPlanetRW p : planet_list)
        p.write(bw);
      bw.write(SStringUtil.END_LINE_CARACTER);
    }
      
    if(!star_list.isEmpty())
    {
      for(SStarRW s : star_list)
        s.write(bw);
      bw.write(SStringUtil.END_LINE_CARACTER);
    }
    
  }
    
  @Override
  public String getRWName()
  {
    return SKeyWordDecoder.KW_SCENE;
  }
  
  @Override
  public String[] getRWParameterName()
  {
    String[] other_parameters = super.getRWParameterName();
    
    return SStringUtil.merge(other_parameters, KEYWORD_PARAMETER);
  }
    
}
