/**
 * 
 */
package sim.readwrite.object;

import java.io.BufferedWriter;
import java.io.IOException;

import sim.readwrite.SAbstractRW;
import sim.util.SBufferedReader;
import sim.util.SInitializationException;
import sim.util.SKeyWordDecoder;
import sim.util.SReadingException;
import sim.util.SStringUtil;

/**
 * La classe <b>SSimulationParameterRW</b> reprsente un paramtre de contrle de simulation pouvant tre lu et crit dans un fichier texte.
 * 
 * @author Simon Vzina
 * @since 2023-09-27
 * @version 2023-10-02
 */
public class SSimulationParameterRW extends SAbstractRW {

  /**
   * La constante <b>KEYWORD_PARAMETER</b> correspond  un tableau contenant l'ensemble des mots cls 
   *  utiliser reconnus lors de la dfinition de l'objet par une lecture en fichier.
   */
  private static final String[] KEYWORD_PARAMETER = {
    SKeyWordDecoder.KW_TIME_STEP, SKeyWordDecoder.KW_NB_STEP, SKeyWordDecoder.KW_NUMERICAL_ALGORITHM
  };
  
  //Type d'algorithm numrique
  public final static String[] NUMERICAL_ALGORITHM = { "euler", "euler_second_order", "RK4"};
  public final static int EULER = 0;
  public final static int EULER_SECOND_ORDER = 1;
  public final static int RK4 = 2;
  
  /**
   * ...
   */
  private static final double DEFAULT_TIME_STEP = 0.01;
  
  /**
   * ...
   */
  private static final int DEFAULT_NB_STEP = 1;
  
  /**
   * ...
   */
  private static final int DEFAULT_NUMERICAL_ALGORITHM = 0;
  
  /**
   * ...
   */
  private double time_step;
  
  /**
   * ...
   */
  private int nb_step;
  
  /**
   * ...
   */
  private int numerical_algorithm;
  
  /**
   * 
   */
  public SSimulationParameterRW() 
  {
    this.time_step = DEFAULT_TIME_STEP;
    this.nb_step = DEFAULT_NB_STEP;
    this.numerical_algorithm = DEFAULT_NUMERICAL_ALGORITHM;
  }

  /**
   * ...
   * 
   * @param sbr
   * @throws SReadingExceltion
   */
  public SSimulationParameterRW(SBufferedReader sbr) throws SReadingException
  {
    this();   
    
    try{
      read(sbr);
    }catch(SInitializationException e){
      throw new SReadingException("Erreur SSimulationParameterRW 001 : Une erreur d'initialisation est survenue.", e);
    }
  }
  
  /**
   * ...
   * 
   * @return
   */
  public double getTimeStep()
  {
    return this.time_step;
  }
  
  /**
   * ...
   * 
   * @return
   */
  public int getNbStep() 
  {
    return this.nb_step;
  }
  
  /**
   * ...
   * 
   * @return
   */
  public int getNumericalAlgorithm()
  {
    return this.numerical_algorithm;
  }
  
  /**
   * ...
   * 
   * @return
   */
  public String getNumericalAlgorithmName()
  {
    return SSimulationParameterRW.NUMERICAL_ALGORITHM[this.numerical_algorithm];
  }
  
  @Override
  public String getRWName() 
  {
    return SKeyWordDecoder.KW_SIMULATION_PARAMETER;
  }

  @Override
  public void writeInformation(BufferedWriter bw) throws IOException
  {
    super.writeInformation(bw);
    
    bw.write(SKeyWordDecoder.KW_TIME_STEP);
    bw.write("\t\t");
    bw.write(Double.toString(this.time_step));
    bw.write(" s");
    bw.write(SStringUtil.END_LINE_CARACTER);
    
    bw.write(SKeyWordDecoder.KW_NB_STEP);
    bw.write("\t\t\t");
    bw.write(Integer.toString(this.nb_step));
    bw.write(" steps");
    bw.write(SStringUtil.END_LINE_CARACTER);
    
    bw.write(SKeyWordDecoder.KW_NUMERICAL_ALGORITHM);
    bw.write("\t");
    bw.write(this.getNumericalAlgorithmName());
    bw.write(SStringUtil.END_LINE_CARACTER);
  }
   
  @Override
  protected void initialization() throws SInitializationException
  {
    
  }
  
  @Override
  protected boolean read(SBufferedReader sbr, int code, String remaining_line) throws SReadingException
  {
    switch(code)
    {
      case SKeyWordDecoder.CODE_TIME_STEP : this.time_step = this.readDoubleGreaterThanZero(remaining_line, SKeyWordDecoder.KW_TIME_STEP); return true;
    
      case SKeyWordDecoder.CODE_NB_STEP : this.nb_step = this.readIntGreaterThanZero(remaining_line, SKeyWordDecoder.KW_NB_STEP); return true;
      
      case SKeyWordDecoder.CODE_NUMERICAL_ALGORITHM : this.numerical_algorithm = readIntOrExpression(remaining_line, SKeyWordDecoder.KW_NUMERICAL_ALGORITHM, SSimulationParameterRW.NUMERICAL_ALGORITHM); return true;
       
      default : return super.read(sbr, code, remaining_line);
    } 
  }
  
  @Override
  public String[] getRWParameterName()
  {
    String[] other_parameters = super.getRWParameterName();
    
    return SStringUtil.merge(other_parameters, KEYWORD_PARAMETER);
  }
  
}
