/**
 * 
 */
package sim.readwrite.object;

import java.io.BufferedWriter;
import java.io.IOException;

import sim.graphics.SColor;
import sim.physics.SStar;
import sim.util.SBufferedReader;
import sim.util.SInitializationException;
import sim.util.SKeyWordDecoder;
import sim.util.SReadingException;
import sim.util.SStringUtil;

/**
 * ...
 * 
 * @author Simon Vzina
 * @since 2023-10-10
 * @version 2023-10-10
 */
public class SStarRW extends SPlanetRW {

  /**
   * La constante <b>KEYWORD_PARAMETER</b> correspond  un tableau contenant l'ensemble des mots cls 
   *  utiliser reconnus lors de la dfinition de l'objet par une lecture en fichier.
   */
  private static final String[] KEYWORD_PARAMETER = {
    SKeyWordDecoder.KW_COLOR
  };
  
  /**
   * ...
   */
  private static final SColor DEFAULT_COLOR = SColor.WHITE;
  
  /**
   * ...
   */
  protected SColor color;
  
  /**
   * 
   */
  public SStarRW()
  {
    super();
    
    this.color = DEFAULT_COLOR;
  }

  /**
   * @param sbr
   * @throws SReadingException
   */
  public SStarRW(SBufferedReader sbr) throws SReadingException 
  {
    this();   
    
    try{
      read(sbr);
    }catch(SInitializationException e){
      throw new SReadingException("Erreur SStarRW 001 : Une erreur d'initialisation est survenue.", e);
    }
  }

  @Override
  public String getRWName()
  {
    return SKeyWordDecoder.KW_STAR;
  }
  
  /**
   * ...
   * 
   * @return
   */
  public SStar toSStar()
  {
    return new SStar(this.mass, this.ray, this.position, this.speed, this.texture_name, this.color);
  }
  
  @Override
  public void writeInformation(BufferedWriter bw) throws IOException
  {
    super.writeInformation(bw);
    
    bw.write(SKeyWordDecoder.KW_COLOR);
    bw.write("\t\t");
    this.color.write(bw);
    bw.write(SStringUtil.END_LINE_CARACTER);
  }
  
  @Override
  protected void initialization() throws SInitializationException
  {
    // rien  faire.
  }
  
  @Override
  protected boolean read(SBufferedReader sbr, int code, String remaining_line) throws SReadingException
  {
    switch(code)
    {
      case SKeyWordDecoder.CODE_COLOR : this.color = this.readSColor(remaining_line, SKeyWordDecoder.KW_COLOR); return true;
      
      default : return super.read(sbr, code, remaining_line);
    } 
  }
  
  @Override
  public String[] getRWParameterName()
  {
    String[] other_parameters = super.getRWParameterName();
    
    return SStringUtil.merge(other_parameters, KEYWORD_PARAMETER);
  }
  
  
}
