/**
 * 
 */
package sim.readwrite.object.light;

import java.io.BufferedWriter;
import java.io.IOException;

import sim.graphics.SColor;
import sim.readwrite.SAbstractRW;
import sim.util.SBufferedReader;
import sim.util.SInitializationException;
import sim.util.SKeyWordDecoder;
import sim.util.SReadingException;
import sim.util.SStringUtil;

/**
 * La classe <b>SAbstractLightRW</b> reprsente une source de lumire abstraite contenant uniquement le champ de la couleur de la source de lumire pouvant tre lu dans un fichier texte.
 * 
 * @author Simon Vzina
 * @since 2023-10-09
 * @version 2023-10-10
 */
public class SAbstractLightRW extends SAbstractRW {

  /**
   * La constante <b>KEYWORD_PARAMETER</b> correspond  un tableau contenant l'ensemble des mots cls 
   *  utiliser reconnus lors de la dfinition de l'objet par une lecture en fichier.
   */
  private static final String[] KEYWORD_PARAMETER = { SKeyWordDecoder.KW_COLOR };
  
  /**
   * La constante <b>DEFAULT_LIGHT_COLOR</b> reprsente la couleur de la source de lumire par dfaut.
   */
  private static final SColor DEFAULT_LIGHT_COLOR = SColor.WHITE;      
    
  protected SColor color;
  
  /**
   * 
   */
  public SAbstractLightRW() 
  {
    super();
    
    this.color = DEFAULT_LIGHT_COLOR;
  }

  /**
   * ...
   * 
   * @param sbr
   * @throws SReadingExceltion
   */
  public SAbstractLightRW(SBufferedReader sbr) throws SReadingException
  {
    this();   
    
    try{
      read(sbr);
    }catch(SInitializationException e){
      throw new SReadingException("Erreur SAbstractLightRW 001 : Une erreur d'initialisation est survenue.", e);
    }
  }

  @Override
  public String getRWName() 
  {
    throw new UnsupportedOperationException("Erreur SAbstractLightRW 002 : Puisque cet objet est abstrait, il ne possde pas de nom.");
  }

  @Override
  public void writeInformation(BufferedWriter bw) throws IOException
  {
    super.writeInformation(bw);
    
    bw.write(SKeyWordDecoder.KW_COLOR);
    bw.write("\t\t");
    color.write(bw);
    bw.write(SStringUtil.END_LINE_CARACTER);
  }
  
  @Override
  protected void initialization() throws SInitializationException
  {
  
  }
  
  @Override
  protected boolean read(SBufferedReader sbr, int code, String remaining_line) throws SReadingException
  {
    switch(code)
    {
      case SKeyWordDecoder.CODE_COLOR : this.color = this.readSColor(remaining_line, SKeyWordDecoder.KW_COLOR); return true;
                   
      default : return super.read(sbr, code, remaining_line);
    } 
  }
  
  @Override
  public String[] getRWParameterName()
  {
    String[] other_parameters = super.getRWParameterName();
    
    return SStringUtil.merge(other_parameters, KEYWORD_PARAMETER);
  }
  
}
