package sim.util;

import sim.exception.SRuntimeException;

/**
 * La classe SChronometer permet d'valuer le temps coul avec une rsolution en milliseconde.
 * @author Simon Vezina
 * @since 2014-12-08
 * @version 2023-10-20
 */
public class SChronometer 
{
    /**
     * La variable <b>start_time</b> reprsente le temps coul en milliseconde depuis 
     * January 1, 1970, 00:00:00 GMT pour signaler le <u>dbut du chronomtre</u>.
     */
	private long start_time;
	
	private boolean isStarted;
	
	/**
	 * La variable <b>stop_time</b> reprsente le temps coul en milliseconde depuis 
	 * January 1, 1970, 00:00:00 GMT pour signaler <u>l'arrt du chronomtre</u>.
	 */
	private long stop_time;	
	
	private boolean isStoped;
	
	/**
	 * Constructeur du chronometre.
	 */
	public SChronometer()
	{
		reset();
	}
	
	/**
	 * Mthode pour initialiser le chronometre.
	 */
	public void start()
	{	
		start_time = System.currentTimeMillis();	// Temps  partir de January 1, 1970, 00:00:00 GMT
		isStarted = true;
	}
	
	/**
	 * Mthode pour arrter la mesure de l'coulement du temps depuis l'appel de la mthode start().
	 */
	public void stop()
	{
		stop_time = System.currentTimeMillis();		// Temps  partir de January 1, 1970, 00:00:00 GMT
		isStoped = true;
	}
	
	/**
	 * Mthode pour rinitialiser le chronomtre.
	 */
	public void reset()
	{
		start_time = 0;
		isStarted = false;
		
		stop_time = 0;
		isStoped = false;
	}
	
	/**
	 * Mthode pour dterminer si le chronomtre a t initialis.
	 * 
	 * @return <b>true</b> si le chronomtre a t intialis et <b>false</b> sinon.
	 */
	public boolean asStart()
	{
	  return isStarted;
	}
	
	/**
	 * Mthode pour obtenir l'coulement du temps entre l'appel de la fonction start() et stop()
	 * ou bien le temps coul entre l'appel de la fonction start() et cette fonction courante.
	 * 
	 * @return Temps coul en seconde.
	 * @throws SRuntimeException S'il l'appel de la mthode start() n'a pas encore initialise le chronomtre.
	 */
	public double getTime()throws SRuntimeException
	{
		return (double) getTimeMilliseconde() / 1000;
	}
	
	/**
	 * Mthode pour obtenir l'coulement du temps entre l'appel de la fonction start() et stop().
	 * ou bien le temps coul entre l'appel de la fonction start() et cette fonction courante.
	 * 
	 * @return Temps coul en milliseconde.
	 * @throws SRuntimeException S'il l'appel de la mthode start() n'a pas encore initialise le chronomtre.
	 */
	public long getTimeMilliseconde()throws SRuntimeException
	{
		if(!isStarted)
			throw new SRuntimeException("Erreur SChronometer 001 : La mthode start() du chronomtre n'a pas encore initialise le chronomtre.");
		
		if(isStoped)
			return stop_time - start_time;
		else
		{
			//Si le chronomtre n'a pas t stopp, retourner le temps coul entre le start() et l'appel de la fonction getTimeMilliseconde()
			long current_time = System.currentTimeMillis();
			return current_time - start_time;
		}
	}

  @Override
  public String toString()
  {
    return "SChronometer [TEMPS COUL : " + getTime() + " s ]";
  }
	
  /**
   * Mthode pour convertir un temps en seconde entire vers un affichage en jour/heure/minutes/seconde
   * 
   * @see https://stackoverflow.com/questions/6118922/convert-seconds-value-to-hours-minutes-seconds
   * @param sec Le temps en seconde.
   * @return Le temps en format anne/jour/heure/minutes/seconde.
   */
  public static String secToTimeFormatYDHHMMSS(int sec) 
  {
    int seconds = sec % 60;
    
    int minutes = sec / 60;
    
    if(minutes >= 60)
    {
        int hours = minutes / 60;
        minutes %= 60;
        
        if(hours >= 24) 
        {
            int days = hours / 24;
            hours = hours % 24;
            
            // On va exclure les annes bissextile.
            if(days > 365)  
            {
              int years = days / 365;
              days = days % 365;
            
              
              if(years > 100)
              {
                int century = years / 100;
                years = years % 100;
                
                // Affichage du format avec sicle
                return String.format("%d century %d years %d days %02d:%02d:%02d", century, years, days, hours, minutes, seconds);
              }
              
              // Affichage du format avec anne.
              return String.format("%d years %d days %02d:%02d:%02d", years, days, hours, minutes, seconds);
            }
                        
            // Affichage du format avec jours.
            return String.format("%d days %02d:%02d:%02d", days, hours, minutes, seconds);
        }
        
        // Affichage du format avec heure
        return String.format("%02d:%02d:%02d", hours, minutes, seconds);
    }
    
    // Affichage du format avec minutes et secondes.
    return String.format("00:%02d:%02d", minutes, seconds);
  }
  
}
