/**
 * 
 */
package sim.util;

import java.io.File;
import java.io.FileNotFoundException;
import java.util.ArrayList;
import java.util.List;
import java.util.Scanner;
import java.util.Set;
import java.util.TreeSet;

/**
 * La classe <b>SFileUtil</b> est une classe utilitaire effectuant des oprations sur des fichiers.
 * 
 * @author Simon Vezina
 * @since 2023-05-31
 * @version 2023-06-11
 */
public class SFileUtil {

  /**
   * <p>
   * Mthode pour extraire un ensemble de mots d'un texte. Puisque c'est un ensemble, il n'y aura pas d'ordre et aucune rptition.
   * Dans cette version, l'ensemble sera <u>tri</u>.
   * </p>
   * <p>
   * Il est important de remarquer que la <u>ponctuation</u> sera retir de l'ensemble.
   * </p>
   * 
   * @param file_name Le nom du fichier.
   * @return L'ensemble des mots.
   * @throws IllegalArgumentException S'il y a un problme avec le nom du fichier.
   */
  public static Set<String> extractSetOfWordFromText(String file_name) throws IllegalArgumentException
  {
    return new TreeSet<String>(SFileUtil.extractListOfWordFromText(file_name));    
  }
  
  /**
   * ...
   * 
   * @param file_name
   * @return
   * @throws IllegalArgumentException
   */
  public static Set<String> extractSetOfWordAndPonctuationFromText(String file_name) throws IllegalArgumentException
  {
    return new TreeSet<String>(SFileUtil.extractListOfWordAndPonctuationFromText(file_name));    
  }
  
  /**
   * <p>
   * Mthode pour extraire une liste de mots d'un texte. Puisque c'est une liste, les mots seront dans l'ordre avec rptition.
   * </p>
   * <p>
   * Il est important de remarquer que la <u>ponctuation</u> sera retir de l'ensemble.
   * </p>
   * 
   * @param file_name Le nom du fichier.
   * @return La liste des mots.
   * @throws IllegalArgumentException S'il y a un problme avec le nom du fichier.
   */
  public static List<String> extractListOfWordFromText(String file_name) throws IllegalArgumentException
  {
    List<String> list = new ArrayList<String>();
    SBrowser browser = new SBrowser();
    File file;
    Scanner scanner;
    
    try {
      
      file = browser.findFile(file_name);
      scanner = new Scanner(file);
      
    }catch(FileNotFoundException e) {
      throw new IllegalArgumentException("Le fichier n'a pas t trouv.", e);
    }catch(SManyFilesFoundException e) {
      throw new IllegalArgumentException("Le fichier a t trouv en plusieurs exemplaire.", e);
    } 
         
    while(scanner.hasNext())
    {
      String w = scanner.next().toLowerCase();
        
      // Analyse du dernier caractre du mot.
      switch(w.charAt(w.length()-1))
      {
        case '.' :
        case ',' :    
        case '?' :
        case '!' :
        case ':' :
        case ';' :
          
          // Construire le mot sans le dernier caractre.
          w = w.substring(0, w.length()-1);
      }
        
      if(!w.equals(SStringUtil.EMPTY_STRING))
        list.add(w);
    }
      
    scanner.close();
      
    return list;
  }
  
  /**
   * ...
   *  
   * @param file_name
   * @return
   * @throws IllegalArgumentException
   */
  public static List<String> extractListOfWordAndPonctuationFromText(String file_name) throws IllegalArgumentException
  {
    List<String> list = new ArrayList<String>();
    
    SBrowser browser = new SBrowser();
    
    File file;
    Scanner scanner;
    
    try {
      
      file = browser.findFile(file_name);
      scanner = new Scanner(file);
      
    }catch(FileNotFoundException e) {
      throw new IllegalArgumentException("Le fichier n'a pas t trouv.", e);
    }catch(SManyFilesFoundException e) {
      throw new IllegalArgumentException("Le fichier a t trouv en plusieurs exemplaire.", e);
    } 
         
    while(scanner.hasNext())
    {
      String w = scanner.next().toLowerCase();
        
      // Analyse du dernier caractre du mot.
      switch(w.charAt(w.length()-1))
      {
        case '.' :
        case ',' :    
        case '?' :
        case '!' :
        case ':' :
        case ';' :
          
          // Capturer le dernier caractre spciale.
          String caracter = Character.toString(w.charAt(w.length()-1));
          
          // Construire le mot sans le dernier caractre.
          w = w.substring(0, w.length()-1);
        
          // Ajouter le mot  la liste s'il n'est pas vide.
          if(!w.equals(SStringUtil.EMPTY_STRING))
            list.add(w);
        
          // Ajouter le dernier caractre spcial.
          list.add(caracter);
          break;
        
        default : 
          
          // Ajouter le mot sans dernier caractre spcial s'il n'est pas vide.
          if(!w.equals(SStringUtil.EMPTY_STRING))
            list.add(w);
      }
        
    }
      
    scanner.close();
      
    return list;
  }
  
}
