package sim.graphics.camera;

import org.junit.Assert;
import org.junit.Test;

import sim.exception.SNoImplementationException;
import sim.math.SVector3d;
import sim.util.SLog;

/**
 * JUnit test permettant de valider les fonctionnalits de la classe <b>SMovableCamera</b>.
 * 
 * @author Simon Vzina
 * @since 2022-05-29
 * @version 2022-06-02
 */
public class SMovableCameraTest {

  /**
   * Test de la mthode <b>moveFront</b> dans le cas d'un dplacement vers l'avant selon l'axe x avec un dplacement positif.
   * Ce test ne ncessite pas l'implmentation du produit vectoriel, car on utilise le constructeur vide de la camra.
   * Dans ce constructeur, la camra regarde dans la direction -x.
   */
  @Test
  public void moveFrontTest1a()
  {
    try {
      
      SVectorCamera camera = new SVectorCamera();
      double distance = 15.0;
            
      SVectorCamera expected = camera.clone();
      expected.setPosition(new SVector3d(camera.getPosition().getX() - distance, 0.0, 0.0));
      
      // Mthode  tester.
      camera.moveFront(distance);
      
      Assert.assertEquals(expected, camera);  
    
    }catch(SNoImplementationException e){ 
      SLog.logWriteLine("SMovableCameraTest ---> Test non effectu : public void moveFrontTest1a()");
    }
  }
  
  /**
   * Test de la mthode <b>moveFront</b> dans le cas d'un dplacement vers l'avant selon l'axe x avec un dplacement ngatif.
   * Ce test ne ncessite pas l'implmentation du produit vectoriel, car on utilise le constructeur vide de la camra.
   * Dans ce constructeur, la camra regarde dans la direction -x.
   */
  @Test
  public void moveFrontTest1b()
  {
    try {
      
      SVectorCamera camera = new SVectorCamera(SVector3d.ORIGIN, SVector3d.UNITARY_X, SVector3d.UNITARY_Z);
      double distance = -15.0;
            
      SVectorCamera expected = camera.clone();
      expected.setPosition(new SVector3d(camera.getPosition().getX() + distance, 0.0, 0.0));
      
      // Mthode  tester.
      camera.moveFront(distance);
      
      Assert.assertEquals(expected, camera);  
    
    }catch(SNoImplementationException e){ 
      SLog.logWriteLine("SMovableCameraTest ---> Test non effectu : public void moveFrontTest1b()");
    }
  }
  
  /**
   * Test de la mthode <b>moveFront</b> dans le cas d'un dplacement vers l'avant selon l'axe x avec un dplacement positif.
   */
  @Test
  public void moveFrontTest2a()
  {
    try {
      
      SVectorCamera camera = new SVectorCamera(SVector3d.ORIGIN, SVector3d.UNITARY_X, SVector3d.UNITARY_Z);
      double distance = 15.0;
            
      SVectorCamera expected = camera.clone();
      expected.setPosition(new SVector3d(distance, 0.0, 0.0));
      
      // Mthode  tester.
      camera.moveFront(distance);
      
      Assert.assertEquals(expected, camera);  
    
    }catch(SNoImplementationException e){ 
      SLog.logWriteLine("SMovableCameraTest ---> Test non effectu : public void moveFrontTest2a()");
    }
  }
  
  /**
   * Test de la mthode <b>moveFront</b> dans le cas d'un dplacement vers l'avant selon l'axe x avec un dplacement ngatif.
   */
  @Test
  public void moveFrontTest2b()
  {
    try {
      
      SVectorCamera camera = new SVectorCamera(SVector3d.ORIGIN, SVector3d.UNITARY_X, SVector3d.UNITARY_Z);
      double distance = -15.0;
            
      SVectorCamera expected = camera.clone();
      expected.setPosition(new SVector3d(distance, 0.0, 0.0));
      
      // Mthode  tester.
      camera.moveFront(distance);
      
      Assert.assertEquals(expected, camera);  
    
    }catch(SNoImplementationException e){ 
      SLog.logWriteLine("SMovableCameraTest ---> Test non effectu : public void moveFrontTest2b()");
    }
  }
  
  /**
   * Test de la mthode <b>move</b> dans le cas d'un dplacement vers l'avant selon l'axe x avec un dplacement positif.
   * Ce test ne ncessite pas l'implmentation du produit vectoriel, car on utilise le constructeur vide de la camra.
   */
  @Test
  public void moveTest1a()
  {
    try {
      
      SVectorCamera camera = new SVectorCamera();
      double distance = 15.0;
            
      SVectorCamera expected = camera.clone();
      expected.setPosition(new SVector3d(camera.getPosition().getX() + distance, 0.0, 0.0));
      
      // Mthode  tester.
      camera.move(SVector3d.UNITARY_X, distance);
      
      Assert.assertEquals(expected, camera);  
    
    }catch(SNoImplementationException e){ 
      SLog.logWriteLine("SMovableCameraTest ---> Test non effectu : public void moveTest1()");
    }
  }
  
  /**
   * Test de la mthode <b>move</b> dans le cas d'un dplacement vers l'avant selon l'axe x avec un dplacement ngatif.
   * Ce test ne ncessite pas l'implmentation du produit vectoriel, car on utilise le constructeur vide de la camra.
   */
  @Test
  public void moveTest1b()
  {
    try {
      
      SVectorCamera camera = new SVectorCamera();
      double distance = -15.0;
            
      SVectorCamera expected = camera.clone();
      expected.setPosition(new SVector3d(camera.getPosition().getX() + distance, 0.0, 0.0));
      
      // Mthode  tester.
      camera.move(SVector3d.UNITARY_X, distance);
      
      Assert.assertEquals(expected, camera);  
    
    }catch(SNoImplementationException e){ 
      SLog.logWriteLine("SMovableCameraTest ---> Test non effectu : public void moveTest1()");
    }
  }
  
  /**
   * Test de la mthode <b>move</b> dans le cas d'un dplacement vers l'avant selon l'axe x avec un dplacement positif.
   */
  @Test
  public void moveTest2()
  {
    try {
      
      SVectorCamera camera = new SVectorCamera(SVector3d.ORIGIN, SVector3d.UNITARY_X, SVector3d.UNITARY_Z);
      double distance = 15.0;
            
      SVectorCamera expected = camera.clone();
      expected.setPosition(new SVector3d(distance, 0.0, 0.0));
      
      // Mthode  tester.
      camera.move(SVector3d.UNITARY_X, distance);
      
      Assert.assertEquals(expected, camera);  
    
    }catch(SNoImplementationException e){ 
      SLog.logWriteLine("SMovableCameraTest ---> Test non effectu : public void moveTest2()");
    }
  }
  
}
