package sim.math;


import static org.junit.Assert.fail;

import org.junit.Assert;
import org.junit.Test;

import sim.exception.SNoImplementationException;
import sim.util.SLog;

/**
 * JUnit test permettant de valider les fonctionnalits de la classe <b>SMatrix4x4</b>.
 * 
 * @author Simon Vzina
 * @since 2015-08-12
 * @version 2022-04-13
 */
public class SMatrix4x4Test {

  /**
   * Test de la mthode <b>get</b> sur une matrice quelconque avec accs valide.
   */
  @Test
  public void getTest1()
  {
    SMatrix4x4 M = new SMatrix4x4(67.0, 59.2, 24.6, 12.3,
                                  56.0, 71.2, 83.6, 37.3,
                                  32.0, 27.2, 51.6, 82.3,
                                  17.0, 15.6, 19.6, 42.3);
    
    // Test avec accs dans la matrice valide.
    Assert.assertEquals(67.0, M.get(0,0), 0.000001);
    Assert.assertEquals(59.2, M.get(0,1), 0.000001);
    Assert.assertEquals(24.6, M.get(0,2), 0.000001);
    
    Assert.assertEquals(83.6, M.get(1,2), 0.000001);
    Assert.assertEquals(37.3, M.get(1,3), 0.000001);
    
    Assert.assertEquals(19.6, M.get(3,2), 0.000001);
    Assert.assertEquals(42.3, M.get(3,3), 0.000001);    
  }
  
  /**
   * Test de la mthode <b>get</b> sur une matrice quelconque dans des cas d'accs dans la matrice hors du commun.
   */
  @Test
  public void getTest2()
  {
    SMatrix4x4 M = new SMatrix4x4(67.0, 59.2, 24.6, 12.3,
                                  56.0, 71.2, 83.6, 37.3,
                                  32.0, 27.2, 51.6, 82.3,
                                  17.0, 15.6, 19.6, 42.3);
        
    // Test pour valider l'accs  l'extrieur des plages valides.
    try {
      double v = M.get(4, 4);
      fail("Ce test est en erreur, car la coordonne (4,4) n'est pas accessible dans une matrice 4x4. La valeur obtenue ici dans la matrice est " + v + ".");
    }catch(ArrayIndexOutOfBoundsException e) {
      // C'est parfait !
    }
    
    // Test pour valider l'accs  l'extrieur des plages valides.
    try {
      double v = M.get(-1, 5);
      fail("Ce test est en erreur, car la coordonne (-1,5) n'est pas accessible dans une matrice 4x4. La valeur obtenue ici dans la matrice est " + v + ".");
    }catch(ArrayIndexOutOfBoundsException e) {
      // C'est parfait !
    }
    
  }
  
  /**
   * Test de la mthode <b>determinant</b> avec la matrice identite.
   * Le dterminant de la matrice identit est gal  1.
   * 
   * Rfrence : https://ncalculators.com/matrix/inverse-matrix.htm
   */
  @Test
  public void determinantTest1()
  {
    SMatrix4x4 M = SMatrix4x4.identity();

    Assert.assertEquals(1.0, M.determinant(), 0.000001);
  }
  
  /**
   * Test de la mthode <b>determinant</b> avec une matrice o le dterminant est 0.
   * 
   * Rfrence : https://ncalculators.com/matrix/inverse-matrix.htm
   */
  @Test
  public void determinantTest2()
  {
    SMatrix4x4 M = new SMatrix4x4(1.0, 2.0, 3.0, 4.0, 
                                  5.0, 6.0, 7.0, 8.0,
                                  9.0, 10.0, 11.0, 12.0,
                                  13.0, 14.0, 15.0, 16.0);

    Assert.assertEquals(0.0, M.determinant(), 0.000001);
  }
  
  /**
   * Test de la mthode <b>determinant</b> avec une matrice symtrique.
   * 
   * Rfrence : https://ncalculators.com/matrix/inverse-matrix.htm
   */
  @Test
  public void determinantTest3()
  {
    SMatrix4x4 M = new SMatrix4x4(1.0, 3.0, 4.0, 5.0, 
                                  3.0, 1.0, 6.0, 7.0,
                                  4.0, 6.0, 1.0, 8.0,
                                  5.0, 7.0, 8.0, 1.0);

    Assert.assertEquals(-1056.0, M.determinant(), 0.000001);
  }
  
  /**
   * Test de la mthode <b>determinant</b> avec une matrice quelconque  terme positif.
   * 
   * Rfrence : https://ncalculators.com/matrix/inverse-matrix.htm
   */
  @Test
  public void determinantTest4()
  {
    SMatrix4x4 M = new SMatrix4x4(1.0, 3.0, 5.0, 11.0, 
                                  14.0, 10.0, 6.0, 7.0,
                                  4.0, 10.0, 1.0, 8.0,
                                  15.0, 18.0, 9.0, 1.0);

    Assert.assertEquals(-8889.0, M.determinant(), 0.000001);
  }
  
  /**
   * ...
   */
  @Test
  public void inverseTest1()
  {
    SMatrix4x4 I = SMatrix4x4.identity();
    
    Assert.assertEquals(SMatrix4x4.identity(), I.inverse());
  }
  
  /**
   * Test de la mthode <b>inverse</b> dans le cas d'une matrice non inversable (car le dterminant est nul).
   * 
   * Rfrence : https://matrix.reshish.com/inverCalculation.php
   */
  @Test
  public void inverseTest2()
  {
    SMatrix4x4 M = new SMatrix4x4(1.0, 2.0, 3.0, 4.0, 
                                  5.0, 6.0, 7.0, 8.0,
                                  9.0, 10.0, 11.0, 12.0,
                                  13.0, 14.0, 15.0, 16.0);
    
    // valuer l'inverse de la matrice.
    try {
      SMatrix4x4 M_inv = M.inverse();       // tentative d'inversion qui devrait tre impossible.
      fail("Le test est en chec, car la matrice en test ne peut pas tre inverse. Voici le rsultat erron : " + M_inv.toString());
    }catch(IllegalArgumentException e) {
      // Le test est bon.
    }   
  }
  
  /**
   * Test de la mthode <b>inverse</b> dans le cas d'une matrice symtrique.
   */
  @Test
  public void inverseTest3()
  {
    SMatrix4x4 M = new SMatrix4x4(1.0, 3.0, 4.0, 5.0, 
                                  3.0, 1.0, 6.0, 7.0,
                                  4.0, 6.0, 1.0, 8.0,
                                  5.0, 7.0, 8.0, 1.0);

    SMatrix4x4 expected = new SMatrix4x4(-524.0/1056.0, 216.0/1056.0, 128.0/1056.0, 84.0/1056.0, 
                                          216.0/1056.0, -216.0/1056.0, 48.0/1056.0, 48.0/1056.0,
                                          128.0/1056.0, 48.0/1056.0, -128.0/1056.0, 48.0/1056.0,
                                          84.0/1056.0, 48.0/1056.0, 48.0/1056.0, -84.0/1056.0);
    
    Assert.assertEquals(expected, M.inverse());
  }
  
  /**
   * Test de la mthode <b>inverse</b> en validant la proprit que le produit d'une matrice avec son inverse donne la matrice identit.
   */
  @Test
  public void inverseIdentityTest()
  {
    // Matrice quelconque dont le dterminant n'est pas nul.
    SMatrix4x4 M = new SMatrix4x4(67.0, 59.2, 24.6, 12.3,
                                  56.0, 71.2, 83.6, 37.3,
                                  32.0, 27.2, 51.6, 82.3,
                                  17.0, 15.6, 19.6, 42.3);
    
    // Test  raliser sur la proprit de la matrice M pour rendre ce test valide.
    Assert.assertFalse(SMath.nearlyZero(M.determinant()));
    
    // Validation de la proprit : M M-1 = I
    Assert.assertEquals(SMatrix4x4.identity(), M.multiply(M.inverse()));
    
    // Validation de la proprit : M-1 M = I
    Assert.assertEquals(SMatrix4x4.identity(),M.inverse().multiply(M));
  }
  
  /**
   * JUnit test permettant de vrifier la prcision numrique de la matrice de rotation
   * selon l'axe x.
   */
  @Test
  public void rotationXTest()
  {
    try {
      
      SVector3d v0 = new SVector3d(0.0, -2.6, -8.3);
      v0 = v0.normalize();
      
      double angle = 90;
      
      SVector3d v1 = SMatrix4x4.rotationX(angle).multiply(v0).getSVector3d();
      
      Assert.assertEquals(0.0, v0.dot(v1), SMath.EPSILON);
    
    }catch(SNoImplementationException e){
      SLog.logWriteLine("SMatrix4x4Test ---> Test non effectu : public void rotationXTest()");
    }
  }
  
  /**
   * JUnit test permettant de vrifier la prcision numrique de la matrice de rotation
   * selon l'axe y.
   */
  @Test
  public void rotationYTest()
  {
    try {
      
      SVector3d v0 = new SVector3d(5.4, 0.0, -8.3);
      v0 = v0.normalize();
      
      double angle = 90;
      
      SVector3d v1 = SMatrix4x4.rotationY(angle).multiply(v0).getSVector3d();
      
      Assert.assertEquals(0.0, v0.dot(v1), SMath.EPSILON);
    
    }catch(SNoImplementationException e){
      SLog.logWriteLine("SMatrix4x4Test ---> Test non effectu : public void rotationYTest()");
    }
  }
  
  /**
   * JUnit test permettant de vrifier la prcision numrique de la matrice de rotation
   * selon l'axe z.
   */
  @Test
  public void rotationZTest()
  {
    try {
      
      SVector3d v0 = new SVector3d(2.4, 5.6, 0.0);
      v0 = v0.normalize();
      
      double angle = 90;
      
      SVector3d v1 = SMatrix4x4.rotationZ(angle).multiply(v0).getSVector3d();
      
      Assert.assertEquals(0.0, v0.dot(v1), SMath.EPSILON);
    
    }catch(SNoImplementationException e){
      SLog.logWriteLine("SMatrix4x4Test ---> Test non effectu : public void rotationZTest()");
    }
  }
  
  @Test
  public void RzyxTest() 
  {
    try{
    
    SVector3d v = new SVector3d(30.0, 40.0, 50.0);
    
    SMatrix4x4 matrix_expected = new SMatrix4x4(0.49240387650610407, -0.456825992585671,   0.7408430568614907,  0.0,
                                                0.5868240888334652,   0.8028723374794714,  0.10504046113295196, 0.0,
                                                -0.6427876096865393,  0.38302222155948895, 0.6634139481689384,  0.0,
                                                0.0,                  0.0,                 0.0,                 1.0);
                                             
    Assert.assertEquals(matrix_expected, SMatrix4x4.Rzyx(v));                                          
       
    }catch(SNoImplementationException e){
      SLog.logWriteLine("SMatrix4x4Test ---> Test non effectu : public void RzyxTest()");
    }
  }

  @Test
  public void RxyzTest() 
  {
    try{
   
    SVector3d v = new SVector3d(30.0, 40.0, 50.0);
    
    SMatrix4x4 matrix_expected = new SMatrix4x4(0.49240387650610407, -0.5868240888334652,   0.6427876096865393,  0.0,
                                                0.8700019037522058,   0.31046846097336755, -0.38302222155948895, 0.0,
                                                0.025201386257487246, 0.7478280708194912,   0.6634139481689384,  0.0,
                                                0.0,                  0.0,                  0.0,                 1.0);

    Assert.assertEquals(matrix_expected, SMatrix4x4.Rxyz(v));
    
    }catch(SNoImplementationException e){
      SLog.logWriteLine("SMatrix4x4Test ---> Test non effectu : public void RxyzTest()");
    }
  }
  
  /**
   * JUnit test valuant la proprit de l'identit associe au produit des matrices Rxyz et Rzyx
   * et la proprit de la commutativit dans le rsultat de ce produit.
   */
  @Test
  public void RxyzAndRzyxIdentityTest() 
  {
    try{
      
    SVector3d v = new SVector3d(30.0, 40.0, 50.0);
    SVector3d inv_v = v.multiply(-1);
    
    Assert.assertEquals(SMatrix4x4.identity(), SMatrix4x4.Rxyz(v).multiply(SMatrix4x4.Rzyx(inv_v)));
    Assert.assertEquals(SMatrix4x4.identity(), SMatrix4x4.Rzyx(v).multiply(SMatrix4x4.Rxyz(inv_v)));
    
    }catch(SNoImplementationException e){
      SLog.logWriteLine("SMatrix4x4Test ---> Test non effectu : public void RxyzAndRzyxIdentityTest()");
    }
  }
  
  /**
   * JUnit test de la mthode <b>Rxyz</b> valuant la proprit du produite de la matrice Rxyz avec Rxyz_T (tant transpose).
   * Le rsultat de ce produit doit donner la matrice identit.
   */
  @Test
  public void RxyzAndTransposeTest()
  {
    try{
      
      SVector3d v = new SVector3d(30.0, 40.0, 50.0);
      
      SMatrix4x4 R = SMatrix4x4.Rxyz(v);
      SMatrix4x4 R_T = R.transpose();
      
      // Validation que le produit de R * R_T donne l'identit.
      Assert.assertEquals(SMatrix4x4.identity(), R.multiply(R_T));
      
    }catch(SNoImplementationException e){
      SLog.logWriteLine("SMatrix4x4Test ---> Test non effectu : public void RxyzAndTransposeTest()");
    }
  }
  
  /**
   * JUnit test de la mthode <b>TrRzyxSc</b> dans un cas gnral.
   */
  @Test
  public void TrRzyxScTest()
  {
    try{
      
    SVector3d scale = new SVector3d(2.3, 4.5, 7.6);
    SVector3d rotation = new SVector3d(30.0, 60.0, -15.0);
    SVector3d translation = new SVector3d(-4.5, 3.2, -5.3);
    
    SMatrix4x4 matrix_expected = new SMatrix4x4(1.1108147002324287,  2.8908090895991276,  4.52226483845811,  -4.5,
                                                -0.2976419018678989, 3.2599996637256057, -5.145786696982827,  3.2,
                                                -1.9918584287042087, 1.125,               3.2908965343808676, -5.3,
                                                0.0,                 0.0,                 0.0,                 1.0);
    
    Assert.assertEquals(matrix_expected, SMatrix4x4.TrRzyxSc(translation, rotation, scale));
   
    }catch(SNoImplementationException e){
      SLog.logWriteLine("SMatrix4x4Test ---> Test non effectu : public void TrRzyxScTest()");
    }
  }
  
  /**
   * JUnit test de la mthode <b>ScRxyzTr</b> dans un cas gnral.
   */
  @Test
  public void ScRxyzTrTest()
  {
    try{
      
    SVector3d scale = new SVector3d(2.3, 4.5, 7.6);
    SVector3d rotation = new SVector3d(30.0, 60.0, -15.0);
    SVector3d translation = new SVector3d(-4.5, 3.2, -5.3);
     
    SMatrix4x4 matrix_expected = new SMatrix4x4(1.1108147002324287, 0.2976419018678989, 1.9918584287042087, -14.603061737200958,
                                                0.8735142772210072, 4.268647069914666,  -1.125,              15.691356376232397,
                                                -6.489289581237268, 2.1952495828140908, 3.290896534380868,   18.7848501483542,
                                                0.0,                0.0,                0.0,                 1.0);

    
    Assert.assertEquals(matrix_expected, SMatrix4x4.ScRxyzTr(scale, rotation, translation));
    
    }catch(SNoImplementationException e){
      SLog.logWriteLine("SMatrix4x4Test ---> Test non effectu : public void ScRxyzTrTest()");
    }
  }
  
  /**
   * JUnit test valuant la proprit de l'identit associe au produit des matrices TrRzyxSc et ScRxyzTr
   * et la proprit de la commutativit dans le rsultat de ce produit.
   */
  @Test
  public void TrRzyxScAndScRxyzTrIdentityTest() 
  {
    try{
      
    SVector3d scale = new SVector3d(2.3, 4.5, 7.6);
    SVector3d scale_inv = new SVector3d(1.0/scale.getX(), 1.0/scale.getY(), 1.0/scale.getZ());
    
    SVector3d translation = new SVector3d(-4.5, 3.2, -5.3);
    SVector3d translation_inv = translation.multiply(-1.0);
    
    SVector3d rotation = new SVector3d(30.0, 60.0, -15.0);
    SVector3d rotation_inv = rotation.multiply(-1.0);
      
    Assert.assertEquals(SMatrix4x4.identity(), SMatrix4x4.TrRzyxSc(translation, rotation, scale).multiply(SMatrix4x4.ScRxyzTr(scale_inv, rotation_inv, translation_inv)));
    Assert.assertEquals(SMatrix4x4.identity(), SMatrix4x4.ScRxyzTr(scale, rotation, translation).multiply(SMatrix4x4.TrRzyxSc(translation_inv, rotation_inv, scale_inv)));
  
    }catch(SNoImplementationException e){
      SLog.logWriteLine("SMatrix4x4Test ---> Test non effectu : public void TrRzyxScAndScRxyzTrIdentityTest");
    }
  }
  
  /**
   * JUnit test permettant de vrifier si la matrice de rotation de type cosine directrice
   * reproduit les matrices de rotation autour des axes fixes x, y et z. 
   */
  @Test
  public void rotationUTest1() 
  {
    try {
      
      SVector3d i = new SVector3d(1.0, 0.0, 0.0);
      SVector3d j = new SVector3d(0.0, 1.0, 0.0);
      SVector3d k = new SVector3d(0.0, 0.0, 1.0);
      
      double angle = 30.0;
      
      Assert.assertEquals(SMatrix4x4.rotationX(angle), SMatrix4x4.rotationU(i, angle));
      Assert.assertEquals(SMatrix4x4.rotationY(angle), SMatrix4x4.rotationU(j, angle));
      Assert.assertEquals(SMatrix4x4.rotationZ(angle), SMatrix4x4.rotationU(k, angle));
      
    }catch(SNoImplementationException e){
      SLog.logWriteLine("SMatrix4x4Test ---> Test non effectu : public void rotationUTest1()");
    }
  }
  
  /**
   * JUnit test permettant de vrifier si la rotation d'un vecteur quelconque sous une
   * matrice de rotation de type cosinue directrice d'un angle de 90o est adquate
   * avec un test de produit scalaire avec deux vecteurs orthogonaux.
   */
  @Test
  public void rotationUTest2() 
  {
    try {
      
      // L'axe de rotation
      SVector3d u = new SVector3d(2.3, 5.7, -2.4);
      u = u.normalize();
      
      // L'angle de rotation de 90o
      double angle = 90;
      
      // Vecteur quelconque  tourner
      SVector3d v0 = new SVector3d(-3.4, 2.3, 8.5);
      v0 = v0.cross(u).normalize();
          
      SVector3d v1 = SMatrix4x4.rotationU(u, angle).multiply(v0).getSVector3d();
      
      Assert.assertEquals(0.0, v0.dot(v1), SMath.EPSILON);
    
    }catch(SNoImplementationException e){
      SLog.logWriteLine("SMatrix4x4Test ---> Test non effectu : public void rotationUTest2()");
    }
  }
  
  /**
   * JUnit test permettant de vrifier si la rotation d'un vecteur quelconque sous une
   * matrice de rotation de type cosinue directrice d'un angle quelconque
   * permet de gnrer un vecteur avec un angle constant par rapport  l'axe de rotation.
   */
  @Test
  public void rotationUTest3() 
  {
    try {
      
      // L'axe de rotation
      SVector3d u = new SVector3d(8.9, -5.4, -6.8);
      u = u.normalize();
      
      // L'angle de rotation
      double angle = 73;
      
      // Vecteur quelconque  tourner
      SVector3d v0 = new SVector3d(-3.4, 8.2, 7.4);
             
      SVector3d v1 = SMatrix4x4.rotationU(u, angle).multiply(v0).getSVector3d();
      
      Assert.assertEquals(u.dot(v0), u.dot(v1), SMath.EPSILON);
    
    }catch(SNoImplementationException e){
      SLog.logWriteLine("SMatrix4x4Test ---> Test non effectu : public void rotationUTest3()");
    }
  }
  
  /**
   * Junit test de la mthode <b>rotationU</b> permettant de vrifier que le produit de R_u * R_u_T (transpos) est gal  la matrice identite.
   */
  @Test
  public void rotationUAndTransposeTest()
  {
    try{
      
      SVector3d u = new SVector3d(30.0, 40.0, 50.0);
      double angle = 38.0;
      
      SMatrix4x4 R = SMatrix4x4.rotationU(u, angle);
      SMatrix4x4 R_T = R.transpose();
      
      // Validation que le produit de R * R_T donne l'identit.
      Assert.assertEquals(SMatrix4x4.identity(), R.multiply(R_T));
      
    }catch(SNoImplementationException e){
      SLog.logWriteLine("SMatrix4x4Test ---> Test non effectu : public void rotationUAndTransposeTest()");
    }
  }
  
  /**
   * Test de la mthode <b>rotationZtoVector</b> dans un cas o n'y aura pas de rotation.
   */
  @Test
  public void rotationZtoVectorTest1()
  {
    // Orientation que l'on veut que z prenne.
    SVector3d u = SVector3d.UNITARY_Z;
    
    // Matrice de rotation  tester.
    SMatrix4x4 R = SMatrix4x4.rotationZtoVector(u);
    
    // Effectuer la rotation de l'axe z  l'aide de la matrice de rotation.
    SVector3d v = R.multiply(SVector3d.UNITARY_Z).getSVector3d();
    
    // VALIDATION : Il faut que u et v soit identique.
    Assert.assertEquals(u, v);
  }
  
  /**
   * Test de la mthode <b>rotationZtoVector</b> dans le cas de la rotation de l'axe z en x.
   */
  @Test
  public void rotationZtoVectorTest2a()
  {
    // Orientation que l'on veut que z prenne.
    SVector3d u = SVector3d.UNITARY_X;
    
    // Matrice de rotation  tester.
    SMatrix4x4 R = SMatrix4x4.rotationZtoVector(u);
    
    // Effectuer la rotation de l'axe z  l'aide de la matrice de rotation.
    SVector3d v = R.multiply(SVector3d.UNITARY_Z).getSVector3d();
    
    // VALIDATION : Il faut que u et v soit identique.
    Assert.assertEquals(u, v);
  }
  
  /**
   * Test de la mthode <b>rotationZtoVector</b> dans le cas de la rotation de l'axe z en -x.
   */
  @Test
  public void rotationZtoVectorTest2b()
  {
    // Orientation que l'on veut que z prenne.
    SVector3d u = SVector3d.UNITARY_X.multiply(-1.0);
    
    // Matrice de rotation  tester.
    SMatrix4x4 R = SMatrix4x4.rotationZtoVector(u);
    
    // Effectuer la rotation de l'axe z  l'aide de la matrice de rotation.
    SVector3d v = R.multiply(SVector3d.UNITARY_Z).getSVector3d();
    
    // VALIDATION : Il faut que u et v soit identique.
    Assert.assertEquals(u, v);
  }
  
  /**
   * Test de la mthode <b>rotationZtoVector</b> dans le cas de la rotation de l'axe z en avec y.
   */
  @Test
  public void rotationZtoVectorTest2c()
  {
    // Orientation que l'on veut que z prenne.
    SVector3d u = SVector3d.UNITARY_Y;
    
    // Matrice de rotation  tester.
    SMatrix4x4 R = SMatrix4x4.rotationZtoVector(u);
    
    // Effectuer la rotation de l'axe z  l'aide de la matrice de rotation.
    SVector3d v = R.multiply(SVector3d.UNITARY_Z).getSVector3d();
    
    // VALIDATION : Il faut que u et v soit identique.
    Assert.assertEquals(u, v);
  }
  
  /**
   * Test de la mthode <b>rotationZtoVector</b> dans un cas quelconque.
   */
  @Test
  public void rotationZtoVectorTest3a()
  {
    // Orientation que l'on veut que z prenne.
    SVector3d u = new SVector3d(2.4, 3.2, 8.0).normalize();
    
    // Matrice de rotation  tester.
    SMatrix4x4 R = SMatrix4x4.rotationZtoVector(u);
    
    // Effectuer la rotation de l'axe z  l'aide de la matrice de rotation.
    SVector3d v = R.multiply(SVector3d.UNITARY_Z).getSVector3d();
    
    // VALIDATION : Il faut que u et v soit identique.
    Assert.assertEquals(u, v);
  }
  
  /**
   * Test de la mthode <b>rotationZtoVector</b> dans un vecteur alatoire de composant <u>strictement positive</u>.
   */
  @Test
  public void rotationZtoVectorTest3b()
  {
    // Orientation que l'on veut que z prenne.
    SVector3d u = new SVector3d(Math.random(), Math.random(), Math.random()).normalize();
    
    // Matrice de rotation  tester.
    SMatrix4x4 R = SMatrix4x4.rotationZtoVector(u);
    
    // Effectuer la rotation de l'axe z  l'aide de la matrice de rotation.
    SVector3d v = R.multiply(SVector3d.UNITARY_Z).getSVector3d();
    
    // Affichage du vecteur en test si le test est en chec.
    if(!u.equals(v))
      System.out.println("Le test rotationZtoVectorTest3b sera en chec avec le vecteur suivant en test : " + u);
    
    // VALIDATION : Il faut que u et v soit identique.
    Assert.assertEquals(u, v);
  }
  
  /**
   * Test de la mthode <b>rotationZtoVector</b> dans un vecteur alatoire de composant <u>strictement ngative</u>.
   */
  @Test
  public void rotationZtoVectorTest3c()
  {
    // Orientation que l'on veut que z prenne.
    SVector3d u = new SVector3d(-Math.random(), -Math.random(), -Math.random()).normalize();
    
    // Matrice de rotation  tester.
    SMatrix4x4 R = SMatrix4x4.rotationZtoVector(u);
    
    // Effectuer la rotation de l'axe z  l'aide de la matrice de rotation.
    SVector3d v = R.multiply(SVector3d.UNITARY_Z).getSVector3d();
    
    // Affichage du vecteur en test si le test est en chec.
    if(!u.equals(v))
      System.out.println("Le test rotationZtoVectorTest3c sera en chec avec le vecteur suivant en test : " + u);
    
    // VALIDATION : Il faut que u et v soit identique.
    Assert.assertEquals(u, v);
  }
  
  /**
   * Test de la mthode <b>rotationZtoVector</b> dans un vecteur totalement alatoire.
   */
  @Test
  public void rotationZtoVectorTest3d()
  {
    double x, y, z;
    
    if(SMath.randomTrueOrFalse())
      x = Math.random();
    else
      x = -1.0*Math.random();
    
    if(SMath.randomTrueOrFalse())
      y = Math.random();
    else
      y = -1.0*Math.random();
    
    if(SMath.randomTrueOrFalse())
      z = Math.random();
    else
      z = -1.0*Math.random();
    
    // Orientation que l'on veut que z prenne.
    SVector3d u = new SVector3d(x, y, z).normalize();
    
    // Matrice de rotation  tester.
    SMatrix4x4 R = SMatrix4x4.rotationZtoVector(u);
    
    // Effectuer la rotation de l'axe z  l'aide de la matrice de rotation.
    SVector3d v = R.multiply(SVector3d.UNITARY_Z).getSVector3d();
    
    // Affichage du vecteur en test si le test est en chec.
    if(!u.equals(v))
      System.out.println("Le test rotationZtoVectorTest3d sera en chec avec le vecteur suivant en test : " + u);
    
    // VALIDATION : Il faut que u et v soit identique.
    Assert.assertEquals(u, v);
  }
  
}//fin de la classe SMatrix4x4Test
