/**
 * 
 */
package sim.physics;

import org.junit.Assert;
import org.junit.Test;

import sim.exception.SNoImplementationException;
import sim.math.SVector3d;
import sim.util.SLog;

/**
 * JUnit test permettant de valider les fonctionnalits de la classe <b>SGravitation</b>.
 * 
 * @author Simon Vzina
 * @since 2023-10-11
 * @version 2024-01-05 (Version labo : Les orbites)
 */
public class SGravitationTest {
  
  /**
   * JUnit test de la mthode <b>gravitationLaw</b> dans le cas de deux masses alignes sur l'axe x  une distance de 1 m dont M  l'origine et m du ct de l'axe x positif.
   */
  @Test
  public void gravitationLawTest1a() 
  {
    try{
      
      double m = 3.34e6;
      double M = 7.41e8;
      
      SVector3d r_m = new SVector3d(1.0, 0.0, 0.0);
      SVector3d r_M = SVector3d.ZERO;
      
      SVector3d expected = new SVector3d(-165162.64596, 0.0, 0.0);
      
      Assert.assertEquals(expected, SGravitation.gravitationLaw(M, r_M, m, r_m));
      
    }catch(SNoImplementationException e){
      SLog.logWriteLine("SGravitationTest ---> Test non effectu : public void gravitationLawTest1a()");
    }
  }

  /**
   * JUnit test de la mthode <b>gravitationLaw</b> dans le cas de deux masses alignes sur l'axe x  une distance de 1 m dont M  l'origine et m du ct de l'axe x ngatif.
   */
  @Test
  public void gravitationLawTest1b() 
  {
    try{
      
      double m = 3.34e6;
      double M = 7.41e8;
      
      SVector3d r_m = new SVector3d(-1.0, 0.0, 0.0);
      SVector3d r_M = SVector3d.ZERO;
      
      SVector3d expected = new SVector3d(165162.64596, 0.0, 0.0);
      
      Assert.assertEquals(expected, SGravitation.gravitationLaw(M, r_M, m, r_m));
      
    }catch(SNoImplementationException e){
      SLog.logWriteLine("SGravitationTest ---> Test non effectu : public void gravitationLawTest1b()");
    }
  }
  
  /**
   * JUnit test de la mthode <b>gravitationLaw/b> dans le cas de deux masses alignes sur l'axe x  une distance de 1 m dont M n'est pas  l'origine.
   */
  @Test
  public void gravitationLawTest1c() 
  {
    try{
      
      double m = 3.34e6;
      double M = 7.41e8;
      
      SVector3d r_m = new SVector3d(9.0, 0.0, 0.0);
      SVector3d r_M = new SVector3d(8.0, 0.0, 0.0);
      
      SVector3d expected = new SVector3d(-165162.64596, 0.0, 0.0);
      
      Assert.assertEquals(expected, SGravitation.gravitationLaw(M, r_M, m, r_m));
      
    }catch(SNoImplementationException e){
      SLog.logWriteLine("SGravitationTest ---> Test non effectu : public void gravitationLawTest1c()");
    }
  }
  
  /**
   * JUnit test de la mthode <b>gravitationLaw/b> dans un cas gnral (backcoverage).
   */
  @Test
  public void gravitationLawTest2() 
  {
    try{
      
      double m = 6.784e12;
      double M = 9.345e16;
      
      SVector3d r_m = new SVector3d(6.7e6, 3.2e6, -4.2e6);
      SVector3d r_M = new SVector3d(-8.8e6, -9.6e6, 7.7e6);
      
      SVector3d expected = new SVector3d(-51441.43433549992, -42480.66835447736, 39493.74636080316);
      
      Assert.assertEquals(expected, SGravitation.gravitationLaw(M, r_M, m, r_m));
      
    }catch(SNoImplementationException e){
      SLog.logWriteLine("SGravitationTest ---> Test non effectu : public void gravitationLawTest2()");
    }
  }
  
  /**
   * JUnit test de la mthode <b>gravitationalEnergy</b> dans un cas gnral o l'on dsire strictement vrifier que l'nergie est ngative.
   */
  @Test
  public void gravitationalEnergyTest1a() 
  {
    try{
      
      double m = 11.3;
      double M = 53.4;
      
      SVector3d r_m = new SVector3d(5.0, 7.0, -9.0);
      SVector3d r_M = new SVector3d(-3.0, -6.0, -12.0);
      
      Assert.assertTrue(SGravitation.gravitationalEnergy(M, r_M, m, r_m) < 0);
      
    }catch(SNoImplementationException e){
      SLog.logWriteLine("SGravitationTest ---> Test non effectu : public void gravitationalEnergyTest1a()");
    }
  }
  
  /**
   * JUnit test de la mthode <b>gravitationalEnergy</b> dans un cas particulier o l'une des particules est situe  l'infini et l'nergie sera gale  zro.
   */
  @Test
  public void gravitationalEnergyTest1b() 
  {
    try{
      
      double m = 11.3;
      double M = 53.4;
      
      SVector3d r_m = new SVector3d(5.4, 7.2, -9.4);
      SVector3d r_M = new SVector3d(Double.NEGATIVE_INFINITY, Double.POSITIVE_INFINITY, Double.POSITIVE_INFINITY);
      
      Assert.assertEquals(0.0, SGravitation.gravitationalEnergy(M, r_M, m, r_m), 1e-10);
      
    }catch(SNoImplementationException e){
      SLog.logWriteLine("SGravitationTest ---> Test non effectu : public void gravitationalEnergyTest1b()");
    }
  }
  
  /**
   * JUnit test de la mthode <b>gravitationalEnergy</b> dans un cas particulier o les deux particules sont situe  la mme position. L'nergie doit tre gale  NEGATIVE_INFINITY.
   */
  @Test
  public void gravitationalEnergyTest1c() 
  {
    try{
      
      double m = 11.3;
      double M = 53.4;
      
      SVector3d r_m = new SVector3d(5.0, 7.0, -9.0);
      SVector3d r_M = r_m;
      
      Assert.assertEquals(Double.NEGATIVE_INFINITY, SGravitation.gravitationalEnergy(M, r_M, m, r_m), 1e-10);
      
    }catch(SNoImplementationException e){
      SLog.logWriteLine("SGravitationTest ---> Test non effectu : public void gravitationalEnergyTest1c()");
    }
  }
  
  /**
   * JUnit test de la mthode <b>gravitationalEnergy</b> dans le cas de deux masses alignes sur l'axe x  une distance de 1 m dont M  l'origine et m du ct de l'axe x positif.
   */
  @Test
  public void gravitationalEnergyTest2a() 
  {
    try{
      
      double m = 3.341e4;
      double M = 7.413e5;
      
      SVector3d r_m = new SVector3d(1.0, 0.0, 0.0);
      SVector3d r_M = SVector3d.ZERO;
      
     Assert.assertEquals(-1.6527898334219997, SGravitation.gravitationalEnergy(M, r_M, m, r_m), 1e-15);
      
    }catch(SNoImplementationException e){
      SLog.logWriteLine("SGravitationTest ---> Test non effectu : public void gravitationalEnergyTest2a()");
    }
  }

  /**
   * JUnit test de la mthode <b>gravitationalEnergy/b> dans le cas de deux masses alignes sur l'axe x  une distance de 1 m dont M n'est pas  l'origine.
   */
  @Test
  public void gravitationalEnergyTest2b() 
  {
    try{
      
      double m = 3.341e4;
      double M = 7.413e5;
      
      SVector3d r_m = new SVector3d(9.0, 0.0, 0.0);
      SVector3d r_M = new SVector3d(8.0, 0.0, 0.0);
      
     Assert.assertEquals(-1.6527898334219997, SGravitation.gravitationalEnergy(M, r_M, m, r_m), 1e-15);
      
    }catch(SNoImplementationException e){
      SLog.logWriteLine("SGravitationTest ---> Test non effectu : public void gravitationalEnergyTest2b()");
    }
  }
  
  /**
   * JUnit test de la mthode <b>gravitationLaw/b> dans un cas gnral (backcoverage).
   */
  @Test
  public void gravitationalEnergyTest3() 
  {
    try{
      
      double m = 6.784e8;
      double M = 9.345e9;
      
      SVector3d r_m = new SVector3d(6.7e6, 3.2e6, -4.2e6);
      SVector3d r_M = new SVector3d(-8.8e6, -9.6e6, 7.7e6);
      
      Assert.assertEquals(-18.11070368831117, SGravitation.gravitationalEnergy(M, r_M, m, r_m), 1e-14);
      
    }catch(SNoImplementationException e){
      SLog.logWriteLine("SGravitationTest ---> Test non effectu : public void gravitationalEnergyTest3()");
    }
  }
  
}
