package sim.physics;

import org.junit.Assert;
import org.junit.Test;

import sim.exception.SNoImplementationException;
import sim.math.SVector3d;
import sim.util.SLog;

/**
 * JUnit test permettant de valider les fonctionnalits de la classe <b>SMechanics</b>.
 * 
 * @author Simon Vezina
 * @since 2020-04-15
 * @version 2023-12-14 (Version laboratoire : Les orbites)
 */
public class SMechanicsTest {

  /**
   * Test de la mthode <b>momentum</b> dans le cas o il n'y a pas de vitesse.
   */
  @Test
  public void momentumTest1a()
  {
    try{
      
      double m = 3.56;
      SVector3d v = SVector3d.ZERO;
      
      Assert.assertEquals(SVector3d.ZERO, SMechanics.momentum(m, v));
      
    }catch(SNoImplementationException e){
      SLog.logWriteLine("SMechanicsTest ---> Test non effectu : public void momentumTest1a()");
    }
  }
  
  /**
   * Test de la mthode <b>momentum</b> dans des cas simples  une dimension. On va tester ici la composante de la vitesse qui doit tre modifi.
   */
  @Test
  public void momentumTest1b()
  {
    try{
      
      double m = 3.56;   
      SVector3d v_x = new SVector3d(6.7, 0.0, 0.0);
      SVector3d v_y = new SVector3d(0.0, -9.4, 0.0);
      SVector3d v_z = new SVector3d(0.0, 0.0, 134.9);
      
      Assert.assertEquals(23.852, SMechanics.momentum(m, v_x).getX(), 0.0001);
      Assert.assertEquals(-33.464, SMechanics.momentum(m, v_y).getY(), 0.0001);
      Assert.assertEquals(480.244, SMechanics.momentum(m, v_z).getZ(), 0.0001);
      
    }catch(SNoImplementationException e){
      SLog.logWriteLine("SMechanicsTest ---> Test non effectu : public void momentumTest1b()");
    }
  }
  
  /**
   * Test de la mthode <b>momentum</b> dans un cas gnral (backcoverage).
   */
  @Test
  public void momentumTest2()
  {
    try{
      
      double m = 3.56312e5;   
      SVector3d v = new SVector3d(5.235, -7.239, -12.82);
      
      SVector3d expected = new SVector3d(1865293.32, -2579342.568, -4567919.84);
     
      Assert.assertEquals(expected, SMechanics.momentum(m, v));
      
    }catch(SNoImplementationException e){
      SLog.logWriteLine("SMechanicsTest ---> Test non effectu : public void momentumTest2()");
    }
  }
  
  /**
   * Test de la mthode <b>kineticEnergy</b> dans le cas o il n'y a pas de vitesse.
   */
  @Test
  public void kineticEnergyTest1a()
  {
    try{
      
      double mass = 3.56;
      SVector3d speed = SVector3d.ZERO;
      
      Assert.assertEquals(0.0, SMechanics.kineticEnergy(mass, speed), 0.00001);
      
    }catch(SNoImplementationException e){
      SLog.logWriteLine("SMechanicsTest ---> Test non effectu : public void kineticEnergyTest1a()");
    }
  }
  
  /**
   * Test de la mthode <b>kineticEnergy</b> dans le cas o la vitesse est seulement dans une direction.
   */
  @Test
  public void kineticEnergyTest1b()
  {
    try{
      
      double m = 5.75;
      
      SVector3d v_x = new SVector3d(6.7, 0.0, 0.0);
      SVector3d v_y = new SVector3d(0.0, -9.4, 0.0);
      SVector3d v_z = new SVector3d(0.0, 0.0, 13.9);
      
      Assert.assertEquals(129.05875, SMechanics.kineticEnergy(m, v_x), 0.0001);
      Assert.assertEquals(254.035, SMechanics.kineticEnergy(m, v_y), 0.0001);
      Assert.assertEquals(555.47875, SMechanics.kineticEnergy(m, v_z), 0.0001);
      
    }catch(SNoImplementationException e){
      SLog.logWriteLine("SMechanicsTest ---> Test non effectu : public void kineticEnergyTest1b()");
    }
  }
  
  /**
   * Test de la mthode <b>kineticEnergy</b> dans le cas gnral (backcoverage).
   */
  @Test
  public void kineticEnergyTest2()
  {
    try{
      
      double m = 5.757e6;
      
      SVector3d v = new SVector3d(-37.92, 23.456, -72.95);
      
      Assert.assertEquals(2.1041296248426003E10, SMechanics.kineticEnergy(m, v), 0.0001);
      
    }catch(SNoImplementationException e){
      SLog.logWriteLine("SMechanicsTest ---> Test non effectu : public void kineticEnergyTest2()");
    }
  }
  
  /**
   * Test de la mthode <b>angularMomentum</b> dans le cas o il n'y a pas de vitesse et lorsque la particule est situe  l'origine.
   */
  @Test
  public void angularMomentumTest1a()
  {
    try{
      
      double m = 3.56;
      SVector3d r = new SVector3d(7.8, -3.4, 9.3);
      SVector3d v = new SVector3d(-94.5, -185.4, 34.8);
      
      // Test avec vitesse nulle.
      Assert.assertEquals(SVector3d.ZERO, SMechanics.angularMomentum(m, r, SVector3d.ZERO));
      
      // Test avec position nulle.
      Assert.assertEquals(SVector3d.ZERO, SMechanics.angularMomentum(m, SVector3d.ZERO, v));
      
    }catch(SNoImplementationException e){
      SLog.logWriteLine("SMechanicsTest ---> Test non effectu : public void angularMomentumTest1a()");
    }
  }
  
  /**
   * Test de la mthode <b>angularMomentum</b> dans des cas simples o le moment cintique sera unitquement selon l'axe z (backcoverage). 
   */
  @Test
  public void angularMomentumTest2()
  {
    try{
      
      double m = 3.5692;   
      SVector3d r_xy = new SVector3d(4.31, 8.92, 0.0);
      SVector3d v_xy = new SVector3d(-8.99, -5.63, 0.0);
      
      Assert.assertEquals(199.609246, SMechanics.angularMomentum(m, r_xy, v_xy).getZ(), 0.0001);
      Assert.assertEquals(0.0, SMechanics.angularMomentum(m, r_xy, v_xy).getX(), 0.0001);
      Assert.assertEquals(0.0, SMechanics.angularMomentum(m, r_xy, v_xy).getY(), 0.0001);
      
    }catch(SNoImplementationException e){
      SLog.logWriteLine("SMechanicsTest ---> Test non effectu : public void angularMomentumTest1b()");
    }
  }
  
  /**
   * Test de la mthode <b>anglarMomentum</b> dans un cas gnral (backcoverage).
   */
  @Test
  public void angularMomentumTest3()
  {
    try{
      
      double m = 3.5692;   
      SVector3d r = new SVector3d(4.31, 8.92, -7.98);
      SVector3d v = new SVector3d(-8.99, -5.63, 8.37);
     
      SVector3d expected = new SVector3d(106.1230236, 127.2973026, 199.6092946);
      
      // En raison du niveau de prcision capricieux dans l'ordre des oprations, nous allons tester composante par composante.
      Assert.assertEquals(expected, SMechanics.angularMomentum(m, r, v));
            
    }catch(SNoImplementationException e){
      SLog.logWriteLine("SMechanicsTest ---> Test non effectu : public void angularMomentumTest3()");
    }
  }
      
}
