package sim.physics;

import org.junit.Assert;
import org.junit.Test;

import sim.exception.SNoImplementationException;
import sim.math.SVector3d;
import sim.util.SLog;

/**
 * JUnit test permettant de valider les fonctionnalits de la classe <b>SNumericalIntegrationMotion</b>.
 * 
 * @author Simon Vezina
 * @since 2023-10-07
 * @version 2023-10-27
 */
public class SNumericalIntegrationMotionTest {

  /**
   * Test de la mthode <b>euler</b> afin de s'assurer que la mthode ne modifie pas les paramtres passe dans la mthode.
   */
  @Test
  public void eulerTest1()
  {
    try {
      
      SVector3d r_i = new SVector3d(4.5, -8.3, -9.1);
      SVector3d v_i = new SVector3d(-7.3, -3.9, 5.2);
      SKineticComponent c_i = new SKineticComponent(r_i, v_i);
      SVector3d a_i = new SVector3d(2.3, -4.9, -7.3);
      double dt = 0.123;
      
      // Excuter la mthode.
      SNumericalIntegrationMotion.euler(c_i, a_i, dt);
      
      // Valeur initiale attendue.
      SKineticComponent expected = new SKineticComponent(r_i, v_i);
      
      // Vrifier que c_i n'a pas t modifi dans la mthode.
      Assert.assertEquals(expected, c_i);
      
    }catch(SNoImplementationException e){
      SLog.logWriteLine("SNumericalIntegrationMotionTest ---> Test non effectu : public void eulerTest1()");
    }
  }
  
  /**
   * Test de la mthode <b>euler</b> dans le cas o il n'y a pas d'acclration et la vitesse est simple.
   */
  @Test
  public void eulerTest2a()
  {
    try {
      
      SVector3d r_i = new SVector3d(4.5, -8.3, -9.1);
      SVector3d v_i = new SVector3d(10.0, 10.0, 10.0);
      SKineticComponent c_i = new SKineticComponent(r_i, v_i);
      SVector3d a_i = SVector3d.ZERO;
      double dt = 0.1;
      
      SKineticComponent expected = new SKineticComponent(new SVector3d(5.5, -7.3, -8.1), v_i);
      
      Assert.assertEquals(expected, SNumericalIntegrationMotion.euler(c_i, a_i, dt));
      
    }catch(SNoImplementationException e){
      SLog.logWriteLine("SNumericalIntegrationMotionTest ---> Test non effectu : public void eulerTest2a()");
    }
  }
  
  /**
   * Test de la mthode <b>euler</b> dans le cas o il n'y a pas d'acclration (backcoverage).
   */
  @Test
  public void eulerTest2b()
  {
    try {
      
      SVector3d r_i = new SVector3d(4.521, -8.375, -9.189);
      SVector3d v_i = new SVector3d(-9.427, 7.517, -8.539);
      SKineticComponent c_i = new SKineticComponent(r_i, v_i);
      SVector3d a_i = SVector3d.ZERO;
      double dt = 0.4;
      
      SKineticComponent expected = new SKineticComponent(new SVector3d(0.7502, -5.3682, -12.6046), v_i);
            
      Assert.assertEquals(expected, SNumericalIntegrationMotion.euler(c_i, a_i, dt));
      
    }catch(SNoImplementationException e){
      SLog.logWriteLine("SNumericalIntegrationMotionTest ---> Test non effectu : public void eulerTest2b()");
    }
  }
  
  /**
   * Test de la mthode <b>euler</b> dans le cas o la vitesse initiale est simplet et l'acclration est simple  traiter.
   */
  @Test
  public void eulerTest3a()
  {
    try {
      
      SVector3d r_i = new SVector3d(4.5, -8.3, -9.1);
      SVector3d v_i = new SVector3d(10.0, 10.0, 10.0);
      SKineticComponent c_i = new SKineticComponent(r_i, v_i);
      SVector3d a_i = new SVector3d(9.3, 4.7, 6.3);
      double dt = 0.1;
      
      SKineticComponent expected = new SKineticComponent(new SVector3d(5.5, -7.3, -8.1), new SVector3d(10.93, 10.47, 10.63));
      
      Assert.assertEquals(expected, SNumericalIntegrationMotion.euler(c_i, a_i, dt));
      
    }catch(SNoImplementationException e){
      SLog.logWriteLine("SNumericalIntegrationMotionTest ---> Test non effectu : public void eulerTest3a()");
    }
  }
  
  /**
   * Test de la mthode <b>euler</b> dans le cas gnral (backcoverage).
   */
  @Test
  public void eulerTest3b()
  {
    try {
      
      SVector3d r_i = new SVector3d(4.5, -8.3, -9.1);
      SVector3d v_i = new SVector3d(-7.3, -3.9, 5.2);
      SKineticComponent c_i = new SKineticComponent(r_i, v_i);
      SVector3d a_i = new SVector3d(2.3, -4.9, -7.3);
      double dt = 0.123;
      
      SKineticComponent expected = new SKineticComponent(new SVector3d(3.6021, -8.7797, -8.4604), new SVector3d(-7.0171, -4.5027, 4.3021));
            
      Assert.assertEquals(expected, SNumericalIntegrationMotion.euler(c_i, a_i, dt));
      
    }catch(SNoImplementationException e){
      SLog.logWriteLine("SNumericalIntegrationMotionTest ---> Test non effectu : public void eulerTest3b()");
    }
  }
  
  /**
   * <p>
   * Test de la mthode <b>eulerForceIntegration</b> dans un cas gnral.
   * </p>
   * <p>
   * Le but de ce test est de s'assurer que l'objet <i>particule</i> ne soit pas modifi durant l'excution de cette mthode.
   * </p>
   */
  @Test
  public void eulerForceIntegrationTest1()
  {
    try {
      
      double m = 5.0;
      SVector3d r_i = new SVector3d(4.521, -8.375, -9.189);
      SVector3d v_i = new SVector3d(-9.427, 7.517, -8.539);
      double dt = 0.2;
      
      SParticle p_before = new SParticle(m, r_i, v_i);
      SParticle p_after = new SParticle(m, r_i, v_i);
      
      
      // Implmentation de l'interface avec une force quelconque constante.
      SApplyForce F = new SApplyForce() {
   
        public SVector3d force(SParticle particle) {
          return new SVector3d(3.5, 7.8, 9.2);
        }
   
      };
      
      // Mthode  tester.
      SNumericalIntegrationMotion.eulerForceIntegration(p_after, F, dt);
      
      // Vrifier que la particule n'a pas t modifi.
      Assert.assertEquals(p_before, p_after);
      
    }catch(SNoImplementationException e){
      SLog.logWriteLine("SNumericalIntegrationMotionTest ---> Test non effectu : public void eulerForceIntegrationTest1()");
    }
  }
  
  /**
   * Test de la mthode <b>eulerForceIntegration</b> dans le o la force est nulle (backcoverage).
   */
  @Test
  public void eulerForceIntegrationTest2()
  {
    try {
      
      double m = 5.0;
      SVector3d r_i = new SVector3d(4.521, -8.375, -9.189);
      SVector3d v_i = new SVector3d(-9.427, 7.517, -8.539);
      double dt = 0.2;
      
      SParticle p = new SParticle(m, r_i, v_i);
      
      // Implmentation de l'interface avec une force quelconque.
      SApplyForce F = new SApplyForce() {
   
        public SVector3d force(SParticle particle) {
          return new SVector3d(0.0, 0.0, 0.0);
        }
   
      };
      
      SKineticComponent expected = new SKineticComponent(new SVector3d(2.6356, -6.8716, -10.8968), v_i);
      
      // Vrifier le rsultat.
      Assert.assertEquals(expected, SNumericalIntegrationMotion.eulerForceIntegration(p, F, dt));
      
    }catch(SNoImplementationException e){
      SLog.logWriteLine("SNumericalIntegrationMotionTest ---> Test non effectu : public void eulerForceIntegrationTest2()");
    }
  }
  
  /**
   * Test de la mthode <b>eulerForceIntegration</b> dans le o la masse est gale  1 kg (backcoverage).
   */
  @Test
  public void eulerForceIntegrationTest3()
  {
    try {
      
      double m = 1.0;
      SVector3d r_i = new SVector3d(4.521, -8.375, -9.189);
      SVector3d v_i = new SVector3d(-9.427, 7.517, -8.539);
      double dt = 0.2;
      
      SParticle p = new SParticle(m, r_i, v_i);
      
      // Implmentation de l'interface avec une force quelconque.
      SApplyForce F = new SApplyForce() {
   
        public SVector3d force(SParticle particle) {
          return new SVector3d(8.9, 3.6, -2.9);
        }
   
      };
      
      // Valeur attendue.
      SKineticComponent expected = new SKineticComponent(new SVector3d(2.6356, -6.8716, -10.8968), new SVector3d(-7.646999999999999, 8.237, -9.119));
            
      // Vrifier le rsultat.
      Assert.assertEquals(expected, SNumericalIntegrationMotion.eulerForceIntegration(p, F, dt));
      
    }catch(SNoImplementationException e){
      SLog.logWriteLine("SNumericalIntegrationMotionTest ---> Test non effectu : public void eulerForceIntegrationTest3()");
    }
  }
  
  /**
   * Test de la mthode <b>eulerForceIntegration</b> dans un cas gnral (backcoverage).
   */
  @Test
  public void eulerForceIntegrationTest4()
  {
    try {
      
      double m = 4.8;
      SVector3d r_i = new SVector3d(4.521, -8.375, -9.189);
      SVector3d v_i = new SVector3d(-9.427, 7.517, -8.539);
      double dt = 0.2;
      
      SParticle p = new SParticle(m, r_i, v_i);
      
      // Implmentation de l'interface avec une force quelconque.
      SApplyForce F = new SApplyForce() {
   
        public SVector3d force(SParticle particle) {
          return new SVector3d(8.9, 3.6, -2.9);
        }
   
      };
      
      // Valeur attendue.
      SKineticComponent expected = new SKineticComponent(new SVector3d(2.6356, -6.8716, -10.8968), new SVector3d(-9.056166666666666, 7.667000000000001, -8.659833333333333));
      
      // Vrifier le rsultat.
      Assert.assertEquals(expected, SNumericalIntegrationMotion.eulerForceIntegration(p, F, dt));
      
    }catch(SNoImplementationException e){
      SLog.logWriteLine("SNumericalIntegrationMotionTest ---> Test non effectu : public void eulerForceIntegrationTest4()");
    }
  }
  
  /**
   * Test de la mthode <b>eulerSecondOrder<b> afin de s'assurer que la mthode ne modifie pas les paramtres passe dans la mthode.
   */
  @Test
  public void eulerSecondOrderTest1()
  {
    try {
      
      SVector3d r_i = new SVector3d(4.521, -8.375, -9.189);
      SVector3d v_i = new SVector3d(-9.427, 7.517, -8.539);
      SKineticComponent c_i = new SKineticComponent(r_i, v_i);
      SVector3d a_i = new SVector3d(3.5, -7.9, -6.2);
      double dt = 0.476;
      
      // Excuter la mthode.
      SNumericalIntegrationMotion.eulerSecondOrder(c_i, a_i, dt);
      
      // Valeur initiale attendue.
      SKineticComponent expected = new SKineticComponent(r_i, v_i);
      
      // Vrifier que c_i n'a pas t modifi dans la mthode.
      Assert.assertEquals(expected, c_i);
      
    }catch(SNoImplementationException e){
      SLog.logWriteLine("SNumericalIntegrationMotionTest ---> Test non effectu : public void eulerSecondOrderTest1()");
    }
  }
  
  /**
   * Test ede la mthode <b>eulerSecondOrder<b> dans un cas sans acclration et avec une vitesse simple.
   */
  @Test
  public void eulerSecondOrderTest2a()
  {
    try {
      
      SVector3d r_i = new SVector3d(4.5, -8.3, -9.1);
      SVector3d v_i = new SVector3d(10.0, 10.0, 10.0);
      SKineticComponent c_i = new SKineticComponent(r_i, v_i);
      SVector3d a_i = SVector3d.ZERO;
      double dt = 0.1;
      
      SKineticComponent expected = new SKineticComponent(new SVector3d(5.5, -7.3, -8.1), v_i);
      
      Assert.assertEquals(expected, SNumericalIntegrationMotion.eulerSecondOrder(c_i, a_i, dt));
      
    }catch(SNoImplementationException e){
      SLog.logWriteLine("SNumericalIntegrationMotionTest ---> Test non effectu : public void eulerSecondOrderTest2a()");
    }
  }
  
  /**
   * Test ede la mthode <b>eulerSecondOrder<b> dans un cas sans acclration (backcoverage).
   */
  @Test
  public void eulerSecondOrderTest2b()
  {
    try {
      
      SVector3d r_i = new SVector3d(4.521, -8.375, -9.189);
      SVector3d v_i = new SVector3d(-9.427, 7.517, -8.539);
      SKineticComponent c_i = new SKineticComponent(r_i, v_i);
      SVector3d a_i = SVector3d.ZERO;
      double dt = 0.4;
      
      SKineticComponent expected = new SKineticComponent(new SVector3d(0.7502, -5.3682, -12.6046), v_i);
            
      Assert.assertEquals(expected, SNumericalIntegrationMotion.eulerSecondOrder(c_i, a_i, dt));
      
    }catch(SNoImplementationException e){
      SLog.logWriteLine("SNumericalIntegrationMotionTest ---> Test non effectu : public void eulerSecondOrderTest2b()");
    }
  }
    
  /**
   * Test ede la mthode <b>eulerSecondOrder<b> avec acclration dans un cas simple : Situation 1 - Section 1.6 (Physique XXI).
   */
  @Test
  public void eulerSecondOrderTest3a()
  {
    try {
      
      SVector3d r_i = new SVector3d(10.0, 0.0, 0.0);
      SVector3d v_i = new SVector3d(6.0, 0.0, 0.0);
      SKineticComponent c_i = new SKineticComponent(r_i, v_i);
      SVector3d a_i = new SVector3d(-2.0, 0.0, 0.0);
      double dt = 1.267949192;
      
      SKineticComponent expected = new SKineticComponent(new SVector3d(15.999999998506546, 0.0, 0.0), new SVector3d(3.464101616, 0.0, 0.0));
            
      Assert.assertEquals(expected, SNumericalIntegrationMotion.eulerSecondOrder(c_i, a_i, dt));
      
    }catch(SNoImplementationException e){
      SLog.logWriteLine("SNumericalIntegrationMotionTest ---> Test non effectu : public void eulerSecondOrderTest3a()");
    }
  }

  /**
   * Test ede la mthode <b>eulerSecondOrder<b> dans un cas avec acclration (backcoverage).
   */
  @Test
  public void eulerSecondOrderTest3b()
  {
    try {
      
      SVector3d r_i = new SVector3d(4.521, -8.375, -9.189);
      SVector3d v_i = new SVector3d(-9.427, 7.517, -8.539);
      SKineticComponent c_i = new SKineticComponent(r_i, v_i);
      SVector3d a_i = new SVector3d(3.5, -7.9, -6.2);
      double dt = 0.476;
      
      SKineticComponent expected = new SKineticComponent(new SVector3d(0.4302560000000001, -5.6918832, -13.9559496), new SVector3d(-7.760999999999999, 3.7566, -11.4902));
     
      Assert.assertEquals(expected, SNumericalIntegrationMotion.eulerSecondOrder(c_i, a_i, dt));
      
    }catch(SNoImplementationException e){
      SLog.logWriteLine("SNumericalIntegrationMotionTest ---> Test non effectu : public void eulerSecondOrderTest3b()");
    }
  }
  
  /**
   * <p>
   * Test de la mthode <b>eulerSecondOrderForceIntegration</b> dans un cas gnral.
   * </p>
   * <p>
   * Le but de ce test est de s'assurer que l'objet <i>particule</i> ne soit pas modifi durant l'excution de cette mthode.
   * </p>
   */
  @Test
  public void eulerSecondOrderForceIntegrationTest1()
  {
    try {
      
      double m = 5.0;
      SVector3d r_i = new SVector3d(4.521, -8.375, -9.189);
      SVector3d v_i = new SVector3d(-9.427, 7.517, -8.539);
      double dt = 0.2;
      
      SParticle p_before = new SParticle(m, r_i, v_i);
      SParticle p_after = new SParticle(m, r_i, v_i);
      
      
      // Implmentation de l'interface avec une force quelconque constante.
      SApplyForce F = new SApplyForce() {
   
        public SVector3d force(SParticle particle) {
          return new SVector3d(3.5, 7.8, 9.2);
        }
   
      };
      
      // Mthode  tester.
      SNumericalIntegrationMotion.eulerSecondOrderForceIntegration(p_after, F, dt);
      
      // Vrifier que la particule n'a pas t modifi.
      Assert.assertEquals(p_before, p_after);
      
    }catch(SNoImplementationException e){
      SLog.logWriteLine("SNumericalIntegrationMotionTest ---> Test non effectu : public void eulerSecondOrderForceIntegrationTest1()");
    }
  }
  
  /**
   * Test de la mthode <b>eulerSecondOrderForceIntegration</b> dans le o la force est nulle (backcoverage).
   */
  @Test
  public void eulerSecondOrderForceIntegrationTest2()
  {
    try {
      
      double m = 5.0;
      SVector3d r_i = new SVector3d(4.521, -8.375, -9.189);
      SVector3d v_i = new SVector3d(-9.427, 7.517, -8.539);
      double dt = 0.2;
      
      SParticle p = new SParticle(m, r_i, v_i);
      
      // Implmentation de l'interface avec une force quelconque.
      SApplyForce F = new SApplyForce() {
   
        public SVector3d force(SParticle particle) {
          return new SVector3d(0.0, 0.0, 0.0);
        }
   
      };
      
      SKineticComponent expected = new SKineticComponent(new SVector3d(2.6356, -6.8716, -10.8968), v_i);
      
      // Vrifier le rsultat.
      Assert.assertEquals(expected, SNumericalIntegrationMotion.eulerSecondOrderForceIntegration(p, F, dt));
      
    }catch(SNoImplementationException e){
      SLog.logWriteLine("SNumericalIntegrationMotionTest ---> Test non effectu : public void eulerSecondOrderForceIntegrationTest2()");
    }
  }
  
  /**
   * Test de la mthode <b>eulerSecondOrderForceIntegration</b> dans le o la masse est gale  1 kg (backcoverage).
   */
  @Test
  public void eulerSecondOrderForceIntegrationTest3()
  {
    try {
      
      double m = 1.0;
      SVector3d r_i = new SVector3d(4.521, -8.375, -9.189);
      SVector3d v_i = new SVector3d(-9.427, 7.517, -8.539);
      double dt = 0.2;
      
      SParticle p = new SParticle(m, r_i, v_i);
      
      // Implmentation de l'interface avec une force quelconque.
      SApplyForce F = new SApplyForce() {
   
        public SVector3d force(SParticle particle) {
          return new SVector3d(8.9, 3.6, -2.9);
        }
   
      };
      
      // Valeur attendue.
      SKineticComponent expected = new SKineticComponent(new SVector3d(2.8136, -6.7996, -10.9548), new SVector3d(-7.646999999999999, 8.237, -9.119));
           
      // Vrifier le rsultat.
      Assert.assertEquals(expected, SNumericalIntegrationMotion.eulerSecondOrderForceIntegration(p, F, dt));
      
    }catch(SNoImplementationException e){
      SLog.logWriteLine("SNumericalIntegrationMotionTest ---> Test non effectu : public void eulerSecondOrderForceIntegrationTest3()");
    }
  }
  
  /**
   * Test de la mthode <b>eulerSecondOrderForceIntegration</b> dans un cas gnral (backcoverage).
   */
  @Test
  public void eulerSecondOrderForceIntegrationTest4()
  {
    try {
      
      double m = 4.8;
      SVector3d r_i = new SVector3d(4.521, -8.375, -9.189);
      SVector3d v_i = new SVector3d(-9.427, 7.517, -8.539);
      double dt = 0.2;
      
      SParticle p = new SParticle(m, r_i, v_i);
      
      // Implmentation de l'interface avec une force quelconque.
      SApplyForce F = new SApplyForce() {
   
        public SVector3d force(SParticle particle) {
          return new SVector3d(8.9, 3.6, -2.9);
        }
   
      };
      
      // Valeur attendue.
      SKineticComponent expected = new SKineticComponent(new SVector3d(2.6726833333333335, -6.8566, -10.908883333333334), new SVector3d(-9.056166666666666, 7.667000000000001, -8.659833333333333));
      
      // Vrifier le rsultat.
      Assert.assertEquals(expected, SNumericalIntegrationMotion.eulerSecondOrderForceIntegration(p, F, dt));
      
    }catch(SNoImplementationException e){
      SLog.logWriteLine("SNumericalIntegrationMotionTest ---> Test non effectu : public void eulerSecondOrderForceIntegrationTest4()");
    }
  }
  
  /**
   * Test de la mthode <b>firstFinalStateRk4</b> afin de s'assurer que la mthode ne modifie pas les paramtres passe dans la mthode.
   */
  @Test
  public void firstFinalStateRK4Test1()
  {
    try {
      
      SVector3d r_i = new SVector3d(4.521, -8.375, -9.189);
      SVector3d v_i = new SVector3d(-9.427, 7.517, -8.539);
      SKineticComponent c_i = new SKineticComponent(r_i, v_i);
      SVector3d a_i = new SVector3d(3.5, -7.9, -6.2);
      double dt = 0.476;
      
      // Excuter la mthode.
      SNumericalIntegrationMotion.firstFinalStateRK4(c_i, a_i, dt);
            
      // Valeur initiale attendue.
      SKineticComponent expected = new SKineticComponent(r_i, v_i);
      
      // Vrifier que c_i n'a pas t modifi dans la mthode.
      Assert.assertEquals(expected, c_i);
      
    }catch(SNoImplementationException e){
      SLog.logWriteLine("SNumericalIntegrationMotionTest ---> Test non effectu : public void firstFinalStateRK4Test1()");
    }
  }
  
  /**
   * Test de la mthode <b>firstFinalStateRk4</b> dans un cas gnral (backcoverage).
   */
  @Test
  public void firstFinalStateRK4Test2()
  {
    try {
      
      SVector3d r_i = new SVector3d(4.521, -8.375, -9.189);
      SVector3d v_i = new SVector3d(-9.427, 7.517, -8.539);
      SKineticComponent c_i = new SKineticComponent(r_i, v_i);
      SVector3d a_i = new SVector3d(3.5, -7.9, -6.2);
      double dt = 0.476;
      
      // Mthode  tester.
      SKineticComponent result = SNumericalIntegrationMotion.firstFinalStateRK4(c_i, a_i, dt);
      
      // Valeur attendue.
      SVector3d expected_position = new SVector3d(0.03374800000000011, -4.796908, -13.253564);
      SVector3d expected_speed = new SVector3d(-7.760999999999999, 3.7566, -11.4902);
            
      // Vrifier la position.
      Assert.assertEquals(expected_position, result.getPosition());
      
      // Vrifier la vitesse.
      Assert.assertEquals(expected_speed, result.getSpeed());
      
      // Vrifier le rsultat.
      Assert.assertEquals(new SKineticComponent(expected_position, expected_speed), result);
      
    }catch(SNoImplementationException e){
      SLog.logWriteLine("SNumericalIntegrationMotionTest ---> Test non effectu : public void firstFinalStateRK4Test2()");
    }
  }
  
  /**
   * Test de la mthode <b>middleTimeStateRK4</b> afin de s'assurer que la mthode ne modifie pas les paramtres passe dans la mthode.
   */
  @Test
  public void middleTimeStateRK4Test1()
  {
    try {
      
      SVector3d r_i = new SVector3d(4.521, -8.375, -9.189);
      SVector3d v_i = new SVector3d(-9.427, 7.517, -8.539);
      SKineticComponent c_i = new SKineticComponent(r_i, v_i);
      SVector3d a_i = new SVector3d(3.5, -7.9, -6.2);
      SVector3d a_f1 = new SVector3d(8.5, -9.9, -7.2);
      double dt = 0.476;
      
      // Excuter la mthode.
      SNumericalIntegrationMotion.middleTimeStateRK4(c_i, a_i, a_f1, dt);
            
      // Valeur initiale attendue.
      SKineticComponent expected = new SKineticComponent(r_i, v_i);
      
      // Vrifier que c_i n'a pas t modifi dans la mthode.
      Assert.assertEquals(expected, c_i);
      
    }catch(SNoImplementationException e){
      SLog.logWriteLine("SNumericalIntegrationMotionTest ---> Test non effectu : public void middleTimeStateRK4Test1()");
    }
  }
  
  /**
   * Test de la mthode <b>middleTimeStateRK4</b> dans un cas gnral (backcoverage).
   */
  @Test
  public void middleTimeStateRK4Test2()
  {
    try {
      
      SVector3d r_i = new SVector3d(4.521, -8.375, -9.189);
      SVector3d v_i = new SVector3d(-9.427, 7.517, -8.539);
      SKineticComponent c_i = new SKineticComponent(r_i, v_i);
      SVector3d a_i = new SVector3d(3.5, -7.9, -6.2);
      SVector3d a_f1 = new SVector3d(8.5, -9.9, -7.2);
      double dt = 0.476;
      
      // Mthode  tester.
      SKineticComponent result = SNumericalIntegrationMotion.middleTimeStateRK4(c_i, a_i, a_f1, dt);
      
      // Valeur attendue.
      SVector3d expected_position = new SVector3d(2.376501, -6.8096978, -11.3968784);
      SVector3d expected_speed = new SVector3d(-8.2965, 5.5178, -10.0741);
           
      // Vrifier la position.
      Assert.assertEquals(expected_position, result.getPosition());
      
      // Vrifier la vitesse.
      Assert.assertEquals(expected_speed, result.getSpeed());
      
      // Vrifier le rsultat.
      Assert.assertEquals(new SKineticComponent(expected_position, expected_speed), result);
      
    }catch(SNoImplementationException e){
      SLog.logWriteLine("SNumericalIntegrationMotionTest ---> Test non effectu : public void middleTimeStateRK4Test2()");
    }
  }
  
  /**
   * Test de la mthode <b>secondFinalStateRK4</b> afin de s'assurer que la mthode ne modifie pas les paramtres passe dans la mthode.
   */
  @Test
  public void secondFinalStateRK4Test1()
  {
    try {
      
      SVector3d r_i = new SVector3d(4.521, -8.375, -9.189);
      SVector3d v_i = new SVector3d(-9.427, 7.517, -8.539);
      SKineticComponent c_i = new SKineticComponent(r_i, v_i);
      SVector3d a_i = new SVector3d(3.5, -7.9, -6.2);
      SVector3d a_f1 = new SVector3d(8.5, -9.9, -7.2);
      double dt = 0.476;
      
      // Excuter la mthode.
      SNumericalIntegrationMotion.secondFinalStateRK4(c_i, a_i, a_f1, dt);
            
      // Valeur initiale attendue.
      SKineticComponent expected = new SKineticComponent(r_i, v_i);
      
      // Vrifier que c_i n'a pas t modifi dans la mthode.
      Assert.assertEquals(expected, c_i);
      
    }catch(SNoImplementationException e){
      SLog.logWriteLine("SNumericalIntegrationMotionTest ---> Test non effectu : public void secondFinalStateRK4Test1()");
    }
  }
  
  /**
   * Test de la mthode <b>secondFinalStateRK4</b> dans un cas gnral (backcoverage).
   */
  @Test
  public void secondFinalStateRK4Test2()
  {
    try {
      
      SVector3d r_i = new SVector3d(4.521, -8.375, -9.189);
      SVector3d v_i = new SVector3d(-9.427, 7.517, -8.539);
      SKineticComponent c_i = new SKineticComponent(r_i, v_i);
      SVector3d a_i = new SVector3d(3.5, -7.9, -6.2);
      SVector3d a_f1 = new SVector3d(8.5, -9.9, -7.2);
      double dt = 0.476;
      
      // Mthode  tester.
      SKineticComponent result = SNumericalIntegrationMotion.secondFinalStateRK4(c_i, a_i, a_f1, dt);
      
      // Valeur attendue.
      SVector3d expected_position = new SVector3d(0.4302560000000001, -5.6918832, -13.9559496);
      SVector3d expected_speed = new SVector3d(-6.571, 3.2806000000000006, -11.7282);
      
      // Vrifier la position.
      Assert.assertEquals(expected_position, result.getPosition());
      
      // Vrifier la vitesse.
      Assert.assertEquals(expected_speed, result.getSpeed());
      
      // Vrifier le rsultat.
      Assert.assertEquals(new SKineticComponent(expected_position, expected_speed), result);
      
    }catch(SNoImplementationException e){
      SLog.logWriteLine("SNumericalIntegrationMotionTest ---> Test non effectu : public void secondFinalStateRK4Test2()");
    }
  }
  
  /**
   * <p>
   * Test de la mthode <b>RK4</b> qui vrifie que la particule utilise dans le calcul de la cinmatique n'a pas t modifie aprs l'excution de la mthode.
   * </p>
   * <p>
   * Dans l'implmentation de la mthode, il est possible que les composantes de la particule ait t modifies temporairement afin d'valuer la force sur celle-ci.
   * Ce test permet alors de vrifier que les paramtres de la particule ont t rtablie.
   * </p>
   */
  @Test
  public void RK4Test1()
  {
    try {
      
      double m = 5.0;
      SVector3d r_i = new SVector3d(4.521, -8.375, -9.189);
      SVector3d v_i = new SVector3d(-9.427, 7.517, -8.539);
      double dt = 0.2;
      
      SParticle p = new SParticle(m, r_i, v_i);
      
      // Implmentation de l'interface avec une force quelconque.
      SApplyForce F = new SApplyForce() {
   
        public SVector3d force(SParticle particle) {
          return new SVector3d(1.0, 2.0, 3.0);
        }
   
      };
      
      // Mthode  tester.
      SNumericalIntegrationMotion.RK4ForceIntegration(p, F, dt);
      
      // Composante initiale attendue aprs l'excution de la mthode  tester.
      SKineticComponent expected_ci = new SKineticComponent(r_i, v_i);
      
      // Vrifier le rsultat.
      Assert.assertEquals(expected_ci, p.getKineticComponent());
      
    }catch(SNoImplementationException e){
      SLog.logWriteLine("SNumericalIntegrationMotionTest ---> Test non effectu : public void RK4Test1()");
    }
    
  }

  /**
   * JUnit test de la mthode <b>RK4</b> sans force et une masse quelconque (backcoverage).
   */
  @Test
  public void RK4Test2()
  {
    try {
      
      double m = 5.0;
      SVector3d r_i = new SVector3d(4.521, -8.375, -9.189);
      SVector3d v_i = new SVector3d(-9.427, 7.517, -8.539);
      double dt = 0.2;
      
      SParticle p = new SParticle(m, r_i, v_i);
      
      // Implmentation de l'interface avec une force quelconque constante.
      SApplyForce F = new SApplyForce() {
   
        public SVector3d force(SParticle particle) {
          return SVector3d.ZERO;
        }
   
      };
      
      // Composante initiale attendue aprs l'excution de la mthode  tester.
      SKineticComponent expected = new SKineticComponent(new SVector3d(2.6356, -6.8716, -10.8968), new SVector3d(-9.427, 7.517, -8.539));
      
      // Mthode  valider.
      Assert.assertEquals(expected, SNumericalIntegrationMotion.RK4ForceIntegration(p, F, dt));
      
    }catch(SNoImplementationException e){
      SLog.logWriteLine("SNumericalIntegrationMotionTest ---> Test non effectu : public void RK4Test2a()");
    }
    
  }
  
  /**
   * <p>
   * JUnit test de la mthode <b>RK4</b> avec une <u>force constante quelconque et une masse de 1 kg</u> (backcoverage).
   * </p>
   * <p>
   * Ce test permet de vrifier si l'usage du calculateur de force est adquat, mais ne vrifie pas si
   * les composants de la particule a t pralablement bien ajusts avant de calculer la force sur celui-ci.
   * </p>
   */
  @Test
  public void RK4Test3a()
  {
    try {
      
      double m = 5.0;
      SVector3d r_i = new SVector3d(4.521, -8.375, -9.189);
      SVector3d v_i = new SVector3d(-9.427, 7.517, -8.539);
      double dt = 0.2;
      
      SParticle p = new SParticle(m, r_i, v_i);
      
      // Implmentation de l'interface avec une force quelconque constante.
      SApplyForce F = new SApplyForce() {
   
        public SVector3d force(SParticle particle) {
          return new SVector3d(7.3, -3.4, 3.0);
        }
   
      };
      
      // Rsultat attendu.
      SKineticComponent expected = new SKineticComponent(new SVector3d(2.6648, -6.8852, -10.8848), new SVector3d(-9.135, 7.381, -8.419));
           
      // Mthode  valider.
      Assert.assertEquals(expected, SNumericalIntegrationMotion.RK4ForceIntegration(p, F, dt));
      
    }catch(SNoImplementationException e){
      SLog.logWriteLine("SNumericalIntegrationMotionTest ---> Test non effectu : public void RK4Test3a()");
    }
    
  }
  
  /**
   * <p>
   * JUnit test de la mthode <b>RK4</b> avec une <u>force constante quelconque et une masse quelconque</u> (backcoverage).
   * </p>
   * <p>
   * Ce test permet de vrifier si l'usage du calculateur de force est adquat, mais ne vrifie pas si
   * les composants de la particule a t pralablement bien ajusts avant de calculer la force sur celui-ci.
   * </p>
   */
  @Test
  public void RK4Test3b()
  {
    try {
      
      double m = 5.62;
      SVector3d r_i = new SVector3d(4.521, -8.375, -9.189);
      SVector3d v_i = new SVector3d(-9.427, 7.517, -8.539);
      double dt = 0.2;
      
      SParticle p = new SParticle(m, r_i, v_i);
      
      // Implmentation de l'interface avec une force quelconque constante.
      SApplyForce F = new SApplyForce() {
   
        public SVector3d force(SParticle particle) {
          return new SVector3d(7.3, -3.4, 3.0);
        }
   
      };
      
      // Rsultat attendu.
      SKineticComponent expected = new SKineticComponent(new SVector3d(2.661578647686833, -6.8836996441281135, -10.886123843416371), new SVector3d(-9.167213523131672, 7.396003558718862, -8.4322384341637));
            
      // Mthode  valider.
      Assert.assertEquals(expected, SNumericalIntegrationMotion.RK4ForceIntegration(p, F, dt));
      
    }catch(SNoImplementationException e){
      SLog.logWriteLine("SNumericalIntegrationMotionTest ---> Test non effectu : public void RK4Test3b()");
    }
  }
  
  /**
   * <p>
   * JUnit test de la mthode <b>RK4</b> avec une force compltement n'importe quoi, mais qui <u>dpend que de la position</u> de la particule (backcoverage).
   * </p>
   * <p>
   * Ce test est plus complet, car l'expression de la force dpend de la position et vitesse de la particule.
   * </p>
   */
  @Test
  public void RK4Test4()
  {
    try {
      
      double m = 5.62;
      SVector3d r_i = new SVector3d(4.521, -8.375, -9.189);
      SVector3d v_i = new SVector3d(-9.427, 7.517, -8.539);
      double dt = 0.2;
      
      SParticle p = new SParticle(m, r_i, v_i);
      
      // Implmentation de l'interface avec une force quelconque, mais qui dpend de la position et vitesse de la particule.
      SApplyForce F = new SApplyForce() {
   
        public SVector3d force(SParticle particle) {
          return particle.getPosition().multiply(3.4);
        }
   
      };
      
      // Rsultat attendu.
      SKineticComponent expected = new SKineticComponent(new SVector3d(2.6828085815360327, -6.96707533651634, -11.015095767699666), new SVector3d(-8.991832165246134, 6.590520790432407, -9.758639434657615));
     
      // Mthode  valider.
      Assert.assertEquals(expected, SNumericalIntegrationMotion.RK4ForceIntegration(p, F, dt));
      
    }catch(SNoImplementationException e){
      SLog.logWriteLine("SNumericalIntegrationMotionTest ---> Test non effectu : public void RK4Test4()");
    }
  }
  
  /**
   * <p>
   * JUnit test de la mthode <b>RK4</b> avec une force compltement n'importe quoi, mais qui <u>dpend des positions et vitesses</u> de la particule (backcoverage).
   * </p>
   * <p>
   * Ce test est plus complet, car l'expression de la force dpend de la position et vitesse de la particule.
   * </p>
   */
  @Test
  public void RK4Test5()
  {
    try {
      
      double m = 5.62;
      SVector3d r_i = new SVector3d(4.521, -8.375, -9.189);
      SVector3d v_i = new SVector3d(-9.427, 7.517, -8.539);
      double dt = 0.2;
      
      SParticle p = new SParticle(m, r_i, v_i);
      
      // Implmentation de l'interface avec une force quelconque, mais qui dpend de la position et vitesse de la particule.
      SApplyForce F = new SApplyForce() {
   
        public SVector3d force(SParticle particle) {
          return particle.getPosition().multiply(3.4).add(particle.getSpeed().multiply(-5.2));
        }
   
      };
      
      // Rsultat attendu.
      SKineticComponent expected = new SKineticComponent(new SVector3d(2.844480799144308, -7.0925957910134265, -10.859196615234007), new SVector3d(-7.432210349944819, 5.397001169131306, -8.206382722819017));
     
      // Mthode  valider.
      Assert.assertEquals(expected, SNumericalIntegrationMotion.RK4ForceIntegration(p, F, dt));
      
    }catch(SNoImplementationException e){
      SLog.logWriteLine("SNumericalIntegrationMotionTest ---> Test non effectu : public void RK4Test5()");
    }
  }
  
}
